/*****************************************************************
*Module Name: filter
*Module Date: 2018.10.24
*Module Auth: chelimin11070377
*****************************************************************/

/* Header File Including ------------------------------------------------------------------------ */
#include "stdio.h"
#include "Filter.h"

/* Function Definition -------------------------------------------------------------------------- */

/*****************************************************************
*Description: reset filter
*Input : *f         filter
*Output: none
*Return: none
*****************************************************************/
void FILTER_reset(FILTER *f) {
    f->reset = 1;
}

/*****************************************************************
*Description: reset fir filter
*Input : *f         fir filter
*Output: none
*Return: none
*****************************************************************/
void FILTER_FIR_reset(FILTER_FIR *f) {
    f->reset = 1;
}

/*****************************************************************
*Description: set filter coefficient
*Input : order      coefficient array length, not filter order
*        *b         b coefficient
*        *a         a coefficient
*        H0         dc signal gain
*Output: *f         filter
*Return: none
*****************************************************************/
void FILTER_setCoef(FILTER *f, int order, double *b, double *a, double H0) {
    int i;

    for (i = 0; i < order; ++i) {
        f->b[i] = b[i];
        f->a[i] = a[i];
    }
    f->order = order;
    f->H0    = H0;
    f->reset = 1;
}

/*****************************************************************
*Description: set fir filter coefficient
*Input : order      coefficient array length, not filter order
*        *b         b coefficient
*        *a         a coefficient
*        H0         dc signal gain
*Output: *f         fir filter
*Return: none
*****************************************************************/
void FILTER_FIR_setCoef(FILTER_FIR *f, int order, double *b, double H0) {
    int i;

    for (i = 0; i < order; ++i) {
        f->b[i] = b[i];
    }
    f->order = order;
    f->H0    = H0;
    f->reset = 1;
}

/*****************************************************************
*Description: filter x signal
*Input : *f         filter
*        x          signal
*Output: none
*Return: filter result
*****************************************************************/
double FILTER_filter(FILTER *f, double x) {
    int i;
    double y;

    if (f->order < 1) {
        printf("fileter order is set to incorrect order, pls chk \
            your code!\n");
        return 0;
    }

    if (f->reset) {
        for (i = 0; i < f->order; ++i) {
            f->yout[i] = f->H0 * x;
            f->xin[i] = x;
        }
        y = f->yout[f->order - 1];
        f->reset = 0;
    }
    // push x and y
    for (i = 0; i < f->order - 1; ++i) {
        f->xin[i] = f->xin[i + 1];
        f->yout[i] = f->yout[i + 1];
    }
    f->xin[f->order - 1] = x;
    // filter
    y = f->b[0] * f->xin[f->order - 1];
    for (i = 1; i < f->order; ++i) {
        y += f->b[i] * f->xin[f->order - 1 - i] - f->a[i] * f->yout[f->order - 1 - i];
    }
    f->yout[f->order - 1] = y;

    return y / f->H0;
}

/*****************************************************************
*Description: filter x signal
*Input : *f         fir filter
*        x          signal
*Output: none
*Return: filter result
*****************************************************************/
double FILTER_FIR_filter(FILTER_FIR *f, double x) {
    int i;
    double y;

    if (f->order < 1) {
        printf("fileter order is set to incorrect order, pls chk \
            your code!\n");
        return 0;
    }

    if (f->reset) {
        for (i = 0; i < f->order; ++i) {
            f->xin[i] = x;
        }
        f->reset = 0;
    }
    // push x
    for (i = 0; i < f->order - 1; ++i) {
        f->xin[i] = f->xin[i + 1];
    }
    f->xin[f->order - 1] = x;
    // filter
    y = f->b[0] * f->xin[f->order - 1];
    for (i = 1; i < f->order; ++i) {
        y += f->b[i] * f->xin[f->order - 1 - i];
    }

    return y / f->H0;
}

void DELAY_reset(DELAYER *d) {
    d->reset = 1;
}

void DELAY_setCoef(DELAYER *d, int delay) {
    d->delay = delay;
    d->reset = 1;
}

double DELAY_delay(DELAYER *d, double x) {
    double yout = d->xin[0];
    int i;

    if (d->reset) {
        for (i = 0; i < d->delay; ++i) {
            d->xin[i] = x;
        }
        yout = x;
        d->reset = 0;
    }
    else {
        for (i = 0; i < d->delay - 1; ++i) {
            d->xin[i] = d->xin[i + 1];
        }
        d->xin[d->delay - 1] = x;
    }

    return yout;
}