/******************** (C) COPYRIGHT 2020 Geek************************************
* File Name          : pdr_kalman.c
* Current Version    : V2.0(compare QCOM SAP 5.0)
* Author             : logzhan
* Date of Issued     : 2020.7.3
* Comments           : PDR ˲
********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#if !defined(LOCATION_PLATFORM_QCOM_MODEM)
#include <malloc.h>
#endif
#include "pdr_base.h"
#include "pdr_kalman.h"
#include "pdr_location.h"
#include "pdr_matrix.h"
#include "pdr_util.h"
#include "pdr_linearFit.h"

#if USE_BUG_FOR_LOCAL_PARA
static double Ekf_Upd_buf_2d[5][N] = { {0.0} };
static double Ekf_Upd_buf_3d[7][N][N] = { { {0.0} } };
#endif

extern double GpsPos[HIST_GPS_NUM][3];

extern int debugCount;
extern double angle_mean[3];

static int hold_type;
static double  b_timestamp;
static double  b_last_time;
static double  attitude_yaw;
static DETECTOR *g_detector;
static int hold_type;
static double BUL_duration;
static double FUR_duration;

/**----------------------------------------------------------------------
* Function    : pdr_initKalman
* Description : ʼ˲
* Date        : 2020/8/27 logzhan
*---------------------------------------------------------------------**/
void pdr_initKalman(void)
{
	b_timestamp  = -1;
	b_last_time  = -1;
	BUL_duration = -1;
	FUR_duration = -1;
	attitude_yaw = -1;
	hold_type = 0;
}

int clear_buf(double *buf_p, int dmt)
{
	int bufLen = 0;
	double *buf_pdr = (double *)buf_p;

	if (NULL == buf_p)return -1;

	if (!(2 == dmt || 1 == dmt)){
		return -1;
	}
	else if (2 == dmt)bufLen = N * N;
	else if (1 == dmt)bufLen = N;

	for (int i = 0; i < bufLen; i++){
		buf_pdr[i] = 0.0f;
	}
	return 0;
}

/**----------------------------------------------------------------------
* Function    : pdr_ekfStatePredict
* Description : pdr˲״̬Ԥⷽ
* Date        : 2020/7/22 logzhan
*---------------------------------------------------------------------**/
void pdr_ekfStatePredict(KfPara *kf, double stepLen, PDR* g_pdr, int step) {

	double (*phi)[N]   = (double(*)[N])&(Ekf_Upd_buf_3d[0][0][0]);
	double (*pvec1)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[1][0][0]);
	double (*pvec2)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[2][0][0]);

	clear_buf((double *)phi, BUF_DMT_1);
	clear_buf((double *)pvec1, BUF_DMT_1);
	clear_buf((double *)pvec2, BUF_DMT_1);

	double deltaHeading = g_pdr->heading - g_pdr->lastHeading;
	double angle        = g_pdr->heading;

	//printf("deltaHeading = %f\n", R2D(deltaHeading));

	//if (fabs(deltaHeading) < D2R(10) && g_pdr->lastHeading != 0.0f) {
	//	angle = deltaHeading + kf->p_xk[3];
	//}else {
	//	angle = g_pdr->heading;
	//}

	if (g_pdr->motionType != PDR_MOTION_TYPE_HANDHELD) {
		deltaHeading = 0;
	}

	angle = deltaHeading + kf->p_xk[3];


	if (fabs(deltaHeading) > 5.5) {
		//printf("%f %f\n", R2D(g_pdr->heading), R2D(g_pdr->lastHeading));
		//printf("d heading = %f\n", R2D(deltaHeading));
		if (deltaHeading < 0) {
			deltaHeading = deltaHeading + D2R(360);
		}else {
			deltaHeading = deltaHeading - D2R(360);
		}
		deltaHeading = 0;
		angle = deltaHeading + kf->p_xk[3];
	}

	if(step == 0){
		kf->p_xk[3] = angle;
	}



    // xk+1 = xk + step * cos(angle)   PDRλø·
	// yk+1 = yk + step * sin(angle)   PDRλø·
	// sk+1 = sk                       һͬ
	// ak+1 = ak * (angle / ak)  ak+1 = ak
	phi[0][0] = 1;            phi[1][0] = 0;          phi[2][0] = 0;  phi[3][0] = 0;
	phi[0][1] = 0;			  phi[1][1] = 1;		  phi[2][1] = 0;  phi[3][1] = 0;
	phi[0][2] = cos(angle);	  phi[1][2] = sin(angle); phi[2][2] = 1;  phi[3][2] = 0;
	phi[0][3] = 0;            phi[1][3] = 0;          phi[2][3] = 0;  phi[3][3] = 1;


	for (uchar i = 0; i < step; i++) {
		// logzhan 21/02/06:
		// phi[3][3]ΪkfԤÿζ¸ֵ, phi[3][3]ÿζ1ڹ
		// ¹УƫǵԤǹ̶ģʵʱûôãifжϿһ̶
		// Ԥֵ£ЧʱûиоԵĺ
		if (fabs(kf->p_xk[3]) > 0.000001 &&
			fabs(angle / kf->p_xk[3]) < 3)
		{
			phi[3][3] = angle / kf->p_xk[3];
		}

		// ״̬·
		// p_xk = phi * p_xk
		VecMatMultiply(kf->p_xk, phi, kf->p_xk);

		// Э : Pk = FPFt + Q
		MatrixMultiply(phi, kf->p_pk, pvec1);               // F*P
		MatrixTrans(phi, pvec2);                        // Ft
		MatrixMultiply(pvec1, pvec2, pvec1);                // F*P*Ft
		MatrixAdd(pvec1, kf->q, kf->p_pk);             // F*P*Ft + Q
	}
}

/**----------------------------------------------------------------------
* Function    : pdr_ekfStateUpdate
* Description : pdr˲״̬·
* Date        : 2020/7/22 logzhan
*---------------------------------------------------------------------**/
void pdr_ekfStateUpdate(KfPara *kf, gnss *pgnss,  classifer *sys, PDR *g_pdr, 
	int scene_type) {

	int j;

	double(*H)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[0][0][0]);
	double *z = &(Ekf_Upd_buf_2d[0][0]);
	double(*I)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[1][0][0]);
	double(*pvec1)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[2][0][0]);
	double(*Ht)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[3][0][0]);
	double(*pvec3)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[4][0][0]);
	double(*pvec4)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[5][0][0]);
	double(*pvec5) = &(Ekf_Upd_buf_2d[1][0]);
	double(*pvec6) = &(Ekf_Upd_buf_2d[2][0]);

	double lambda = 0;
	double(*pv1) = &(Ekf_Upd_buf_2d[3][0]);
	double(*pv2) = &(Ekf_Upd_buf_2d[4][0]);
	double(*pv3)[N] = (double(*)[N])&(Ekf_Upd_buf_3d[6][0][0]);

	clear_buf((double *)z, BUF_DMT_1);
	clear_buf((double *)pvec5, BUF_DMT_1);
	clear_buf((double *)pvec6, BUF_DMT_1);
	clear_buf((double *)pv1, BUF_DMT_1);
	clear_buf((double *)pv2, BUF_DMT_1);

	clear_buf((double *)H, BUF_DMT_2);
	clear_buf((double *)I, BUF_DMT_2);
	clear_buf((double *)pvec1, BUF_DMT_2);
	clear_buf((double *)Ht, BUF_DMT_2);
	clear_buf((double *)pvec3, BUF_DMT_2);
	clear_buf((double *)pvec4, BUF_DMT_2);
	clear_buf((double *)pv3, BUF_DMT_2);


	for (char i = 0; i < 4; i++) {
		H[i][i] = 1;
		I[i][i] = 1;
	}

	// ȡ۲
	z[0] = pgnss->xNed;                                          // λ
	z[1] = pgnss->yNed;                                          // λ   
	z[2] = 0.5;                                                  // Ĭ0.5

	// ifʵʹGPSλñ仯Ͳ仯Ʋ, ⲿִ뻹û
	// ãʵڲ
	//if (pgnss->error > 0 && pgnss->error < 3.8 && pgnss->HDOP <= 0.4 && 
	//	g_pdr->deltaStepsPerGps != 0 && g_pdr->deltaStepsPerGps < 5) {
	//	// GPSٶȺͲƵƲ
	//	//if (g_pdr->motionFreq > 2.5 && (sys->type == 1)) {
	//	//	
	//	//	//printf("z[2] = %f\n", z[2]);
	//	//}
	//	z[2] = pgnss->vel * 0.514 / (g_pdr->deltaStepsPerGps);
	//}else {
	//	z[2] = kf->p_xk[2];
	//}

	z[3] = pgnss->yaw * PI / 180;                                // ۲ĺΪGPSĺ

	//  K = K = PHt / (R + H*P*H^T)
	MatrixTrans(H, Ht);                                              // Ht
	MatrixMultiply(H, kf->p_pk, pvec1);                                  // HP
	MatrixMultiply(pvec1, Ht, pvec3);                                    // pvec3 = H*P*H^T
	MatrixAdd(pvec3, kf->r, pvec4);                                 // pvec4 = R + H*P*H^T
	MatrixInverse(pvec4, pvec1);                                     //  1 / (R + H*P*H^T)

	for (char i = 0; i < N; i++) {
		for (uchar j = 0; j < N; j++) {
			pv3[i][j] = pvec3[i][j];
		}
	}
	MatrixMultiply(kf->p_pk, Ht, pvec3);                                  // PHt
	MatrixMultiply(pvec3, pvec1, kf->Kk);                                 // K = PHt / (R + H*P*H^T)
								                                  
								                                  
	VecMatMultiply(kf->p_xk, H, pvec5);                                   // pvec5 = Hx'
	VectorSub(z, pvec5, pvec6);                                       // pvec6 = Z - Hx'
	modAngle(&pvec6[3], -PI, PI);
	for (char i = 0; i < N; i++) {
		pv1[i] = pvec6[i];
	}
	
	VecMatMultiply(pvec6, kf->Kk, pvec5); // pvec5 = K*( z - Hx')
	// Get the optimally updated state estimation
	VectorAdd(kf->p_xk, pvec5, kf->xk);

	// ---- prepare the matrix for updated estimate covariance ------
	// pvec1 = K*H
	MatrixMultiply(kf->Kk, H, pvec1);
	// pvec2 = (I - K*H)
	MatrixSub(I, pvec1, Ht);
	// pvec3 = (I - K*H)*P
	MatrixMultiply(Ht, kf->p_pk, pvec3);
	// pvec4 = (I- K*H)^T
	MatrixTrans(Ht, pvec4);
	// pvec1 = (I - K*H)*P*(I- K*H)^T
	MatrixMultiply(pvec3, pvec4, pvec1);

	// pvec2 = K*R
	MatrixMultiply(kf->Kk, kf->r, Ht);
	// pvec3 = K^T
	MatrixTrans(kf->Kk, pvec3);
	// pvec4 = K*R*K^T
	MatrixMultiply(Ht, pvec3, pvec4);
	// Get the updated estimate covariance: P' = (I - K*H)*P*(I- K*H)^T + K*R*K^T
	MatrixAdd(pvec1, pvec4, kf->pk);

	// pv2 = (z - Hx')*( H*P*H^T )
	VecMatMultiply(pv1, pv3, pv2);
	// inner product the pre-fit residual and the transitioned residual: lamda = (z - Hx')*( H*P*H^T )  .*  (z-Hx')
	// Calculate the difference between original differenc and the transitioned,
	// if the value is close to zero, it shows that there is a big gap in the 
	// oprimization. Thus, we believe our PDR algorithm
	for (char i = 0; i < N; i++) {
		lambda += pv2[i] * pv1[i];
	}

	kf->lambda = lambda;
	kf->plat = pgnss->lat;
	kf->plon = pgnss->lon;

	if (lambda >= 200)return;

	for (char i = 0; i < N; i++) {
		kf->xk[i] = kf->p_xk[i];
		for (j = 0; j < N; j++) {
			kf->pk[i][j] = kf->p_pk[i][j];
		}
	}
	
}

void calStepLastTime(StepPredict *stepPredict, double timestamp, unsigned long steps_last, 
	unsigned long steps)
{
	float tmpTime = 0;
	if (stepPredict->lastTime > 0 && stepPredict->deltaStep == 0 && (steps - steps_last) > 0)
	{
		tmpTime = (float)((timestamp - stepPredict->lastTime) / (steps - steps_last));
		if (tmpTime > 300 && tmpTime < 800)
		{
			if (stepPredict->meanTime > 0){
				stepPredict->meanTime = stepPredict->meanTime * 0.5f + tmpTime * 0.5f;
			}else{
				stepPredict->meanTime = tmpTime;
			}
		}
	}

	stepPredict->fnum = 0;
	stepPredict->fsum = 0;

	stepPredict->lastTime = timestamp;
	stepPredict->deltaStep = 0;

}

int predictStep(StepPredict *stepPredict, double timestamp, unsigned long steps_last, float gnssVel)
{
	int step = 0;
	float mean_time = 400;
	double dis = 0;

	if (stepPredict->meanTime > 0)
		mean_time = stepPredict->meanTime;

	if ((steps_last > 0) && (stepPredict->fnum > 0) && (timestamp - stepPredict->lastTime > mean_time * 3) && (stepPredict->lastTime > 0))
	{
		stepPredict->fsum = stepPredict->fsum / stepPredict->fnum;		
		dis = sqrt(pow(GpsPos[HIST_GPS_NUM - 1][0]- GpsPos[HIST_GPS_NUM - 2][0], 2) + pow(GpsPos[HIST_GPS_NUM - 1][1] - GpsPos[HIST_GPS_NUM - 2][1], 2)
			 + pow(GpsPos[HIST_GPS_NUM - 1][2] - GpsPos[HIST_GPS_NUM - 2][2], 2));

		if (stepPredict->fsum != 0.0 && (gnssVel > 1.0 || (dis > 1.0 && dis < 3.0)))
		{
			step = (int)((timestamp - stepPredict->lastTime) / mean_time);
			stepPredict->deltaStep += step;
		}

		stepPredict->lastTime = timestamp;
		stepPredict->fnum = 0;
		stepPredict->fsum = 0;
	}

	return step;
}

/**----------------------------------------------------------------------
* Function    : stateRecognition
* Description : ݼٶȼжϵǰ״̬Ƿƽ 1: ƽ  0: ƽ
* Date        : 2020/7/22 logzhan
*---------------------------------------------------------------------**/
void stateRecognition(float *acc, classifer *sys) {
	int i,j,k;
	float accNorm = 0;
	float sum = 0, max_min = 0, max = -1, min = 999;
	float mean = 0, std = 0;

	// ȼ㵱ǰٶȵģ
	accNorm = (float)sqrt(vivopow(acc[0], 2) + vivopow(acc[1], 2) + vivopow(acc[2], 2));

	// 
	for (i = 0; i < PATTERN_RECOGNITION_LEN - 1; i++) {
		sys->accBuffer[i] = sys->accBuffer[i + 1];
	}
	sys->accBuffer[PATTERN_RECOGNITION_LEN - 1] = accNorm;

	// Сֵsum
	for (j = 0; j < PATTERN_RECOGNITION_LEN - 1; j++) {
		sum += sys->accBuffer[j];
		if (sys->accBuffer[j] > max) {
			max = sys->accBuffer[j];
		}
		if (sys->accBuffer[j] < min) {
			min = sys->accBuffer[j];
		}
	}

	// ֵ
	mean = sum / PATTERN_RECOGNITION_LEN;
	// 㷽
	for (k = 0; k < PATTERN_RECOGNITION_LEN - 1; k++) {
		std += (sys->accBuffer[k] - mean) * (sys->accBuffer[k] - mean) / PATTERN_RECOGNITION_LEN;
	}
	std = (float)sqrt(std);

	// Сֵ
	max_min = max - min;

	if (std > 5 && max_min > 25) {
		sys->type = 1;
	}
	else {
		sys->type = 0;
	}
}

/**----------------------------------------------------------------------
* Function    : calPredAngle
* Description : pdr->cal_directiongps_yawǶۺȷǶangle
* Date        : 2020/7/22 logzhan
*---------------------------------------------------------------------**/
double calPredAngle(PDR *g_pdr, classifer *sys, double gpsYaw, double trackYaw, imu *ss_data) {
	int attitude_flag = PDR_FALSE;
	g_detector = pdr_getDetector();

	/*̬YAWstart*/
	if (DETECTOR_TYPE_HANDHELD == g_detector->type)
	{
		b_timestamp = ss_data->gyr.time;
		if (!(angle_mean[0] == 0 && angle_mean[1] == 0 && angle_mean[2] == 0))
		{
			if (b_last_time > 0 && b_timestamp - b_last_time >= 5000 && gpsYaw != 0.0 && gpsYaw != -1 && trackYaw != -1)
			{
				/* ̬yawǵ02*PI */

				/*۰ͷϣ or ұ۰ͷϣ*/
				if (ss_data->acc.s[1] > 9 && (fabs(angle_mean[2] + 6.15 - gpsYaw) < PI / 18 && fabs(angle_mean[2] + 6.15 - trackYaw) < PI / 18))
				{
					FUR_duration = -1;
					if (BUL_duration < 0)
					{
						BUL_duration = ss_data->gyr.time;
					}
					if (BUL_duration >= 0 && (b_timestamp - BUL_duration) >= 1500)
					{
						attitude_yaw = angle_mean[2] + 6.15 + PI;
						attitude_flag = PDR_TRUE;
						hold_type = BACKWARD_UP_LEFT;
					}
				}

				/*ұ۰ͷϣ or ۰ͷϣ*/
				if (ss_data->acc.s[1] > 9 && (fabs(angle_mean[2] + 7.10 - gpsYaw) < PI / 18 && fabs(angle_mean[2] + 7.10 - trackYaw) < PI / 18))
				{
					BUL_duration = -1;
					if (FUR_duration < 0)
					{
						FUR_duration = ss_data->gyr.time;
					}
					if (FUR_duration >= 0 && (b_timestamp - FUR_duration) >= 1500)
					{
						attitude_yaw = angle_mean[2] + 7.10;
						attitude_flag = PDR_TRUE;
						hold_type = FORWARD_UP_RIGHT;
					}
				}
			}
			if (-1 == b_last_time)
			{
				b_last_time = b_timestamp;
			}
		}
	}
	else{
		b_last_time = -1;
		FUR_duration = -1;
		BUL_duration = -1;
	}
	/*̬YAWend*/

	double angle = -1;
	double ahrsYaw = atan2(g_pdr->cal_direction[1], g_pdr->cal_direction[0]);

	//printf("PCA Yaw = %f\n", ahrsYaw * (180 / 3.1415926));

	if (ahrsYaw < 0) {
		ahrsYaw += TWO_PI;
	}
	
	// PCA㷽ǵ׼ȷ
	if ((g_pdr->bias_accuracy > 0 && g_pdr->bias_accuracy < 0.9) && (g_pdr->pca_accuracy > 0 && g_pdr->pca_accuracy < 0.25)) {
		angle = ahrsYaw;
		//printf("PCA׼ȷ, angle = ahrsYaw %f\n", ahrsYaw * (180 / 3.1415926));
	}
	if (gpsYaw != 0.0 && gpsYaw != -1 ) //&& scene_type
	{
		// Status -- Explosion
		if (fabs(ss_data->acc.s[0]) > 70 || fabs(ss_data->acc.s[1]) > 70 || fabs(ss_data->acc.s[2]) > 70 || fabs(ss_data->gyr.s[0]) > 30 || fabs(ss_data->gyr.s[1]) > 30 || fabs(ss_data->gyr.s[2]) > 30) {
			angle = gpsYaw;
			//printf("Explosion, angle = gpsYaw = %f\n", gps_yaw * (180 / 3.1415926));
		}
		// Status -- Running 

		if (g_pdr->motionFreq > 2.7 || (g_pdr->motionType != 3 && g_pdr->motionType != 2) || (sys->type == 1)) { //&& g_pdr->motion_type != 2
			angle = gpsYaw;
			//printf("Running, angle = gpsYaw = %f\n", gps_yaw * (180 / 3.1415926));
		}
	}

	if (trackYaw != -1 && ((g_pdr->bias_accuracy < 0 || g_pdr->bias_accuracy > 0.05) || (g_pdr->pca_accuracy < 0 || g_pdr->pca_accuracy > 0.05))) {
		angle = trackYaw;//gps_yaw: obtained from the datasheet;     G_yaw : calculated from the route
		if (attitude_flag == PDR_TRUE)
		{
			angle = attitude_yaw;
		}
	}

	if (angle < 0)
	{
		if (g_pdr->bias_accuracy > 0 && g_pdr->pca_accuracy > 0 && (g_pdr->bias_accuracy < 1.5 && g_pdr->pca_accuracy < 0.5))//g_pdr->bias_accuracy < 6
		{
			angle = ahrsYaw;
		}
		else if (attitude_flag == PDR_TRUE)
		{
			angle = attitude_yaw;
		}
		else if (gpsYaw != 0.0 && gpsYaw != -1)
		{
			angle = gpsYaw;
		}
		else //if (lastYaw > 0)
		{
			angle = -1;  //lastYaw;
		}
	}

	return angle;
}



double calGnssTrackHeading(double gpsPos[][3], gnss pgnss)
{
	int i;
	double lla[3] = { 0.0 };
	double ned1[3] = { 0.0 };
	double temp[2][HIST_GPS_NUM] = { { 0.0 } };
	double angle[HIST_GPS_NUM - 1] = { 0.0 };

	double ned2[3] = { 0.0 };
	double maxn = 0;
	double maxe = 0;
	double minn = 0;
	double mine = 0;
	double meanx = 0;
	double meany = 0;

	float error = 0;
	float sst = 0;
	float r2 = 0;

	double range = 0;
	double yaw = 0;
	double tempYaw = 0;
	double diffAngle = 0;

	double para[3] = { 0 };
	int flg = 0;

	lla[0] = pgnss.lat;
	lla[1] = pgnss.lon;

	for (i = 0; i < HIST_GPS_NUM - 1; i++)
	{
		gpsPos[i][0] = gpsPos[i + 1][0];
		gpsPos[i][1] = gpsPos[i + 1][1];
		gpsPos[i][2] = gpsPos[i + 1][2];
	}

	if ((lla[0] > 0) && (lla[1] > 0))
	{
		//γWGS84תĵعϵECEF
		WGS842ECEF(lla, gpsPos[HIST_GPS_NUM - 1]);
	}
	else
	{
		gpsPos[HIST_GPS_NUM - 1][0] = 0;
		gpsPos[HIST_GPS_NUM - 1][1] = 0;
		gpsPos[HIST_GPS_NUM - 1][2] = 0;
		return -1;
	}
	if (pgnss.error > 40)
	{
		return -1;
	}

	for (i = 0; i < HIST_GPS_NUM - 1; i++)
	{
		if ((fabs(gpsPos[i][0]) < 1e-6) && (fabs(gpsPos[i][1]) < 1e-6) && (fabs(gpsPos[i][2]) < 1e-6))
		{
			return -1;
		}
	}
	//ĵعϵECEFתϵNED
	ECEF2Ned(gpsPos[HIST_GPS_NUM - 1], lla, ned1);

	for (i = HIST_GPS_NUM - 2; i >= 0; i--) {

		ECEF2Ned(gpsPos[i], lla, ned2);

		temp[0][i] = ned1[0] - ned2[0];
		temp[1][i] = ned1[1] - ned2[1];

		meanx += temp[0][i] / HIST_GPS_NUM;
		meany += temp[1][i] / HIST_GPS_NUM;

		if (maxn > temp[0][i])
		{
			maxn = temp[0][i];
		}
		if (minn < temp[0][i])
		{
			minn = temp[0][i];
		}
		if (maxe > temp[1][i])
		{
			maxe = temp[1][i];
		}
		if (mine < temp[1][i])
		{
			mine = temp[1][i];
		}

		angle[i] = atan2(temp[1][i], temp[0][i]);
		if (angle[i] < 0)
			angle[i] += TWO_PI;
	}
	flg = leastDistanceLinearFit(temp[0], temp[1], HIST_GPS_NUM, para);
	if (flg < 0)
		return -1;

	for (i = HIST_GPS_NUM - 1; i >= 0; i--) {
		error += (float)(vivopow(para[0] * temp[0][i] + para[1] * temp[1][i] + para[2], 2) / (vivopow(para[0], 2) + vivopow(para[1], 2)));
		sst += (float)(vivopow(meanx - temp[0][i], 2) + vivopow(meany - temp[1][i], 2));
	}

	if (sst > 0)
	{
		r2 = 1 - error / sst;
	}
	range = sqrt(vivopow(maxn - minn, 2) + vivopow(maxe - mine, 2));
	//printf("%f,%f,%f\n", r2, error, range);
	if (r2 < 0.95 || range < 1.5 || error > 3.0)
		return -1;

	yaw = atan(-para[0] / para[1]);
	modAngle(&yaw, 0, TWO_PI);

	tempYaw = meanAngle(angle, HIST_GPS_NUM - 1);
	modAngle(&tempYaw, 0, TWO_PI);

	diffAngle = tempYaw - yaw;
	modAngle(&diffAngle, -PI, PI);

	if (fabs(diffAngle) > PI / 2)
		yaw += PI;
	modAngle(&yaw, 0, TWO_PI);

	return yaw;
}

/**----------------------------------------------------------------------
* Function    : pdr_insPredict
* Description : PDRߵλø,
* Date        : 2020/07/08 logzhan
*               2021/02/06 logzhan : оƲƽʱ
*               PDRǰλò
*---------------------------------------------------------------------**/
void  pdr_insPredict(PDR *g_pdr, StepPredict *stepPredict, imu *ss_data, 
	gnss *pgnss, KfPara *kf) {
	int step;
	long deltaStep;
	double stepLength = 0.7;

	stepPredict->fnum++;
	stepPredict->fsum += g_pdr->motionFreq;

	ulong  steps     = g_pdr->steps;
	ulong  lastSteps = g_pdr->lastSteps;
	
	if (steps - lastSteps > 0 && g_pdr->motionFreq != 0.0) {
		deltaStep = steps - lastSteps - stepPredict->deltaStep;

		// ӦǷֹƲ쳣һμƲر
		if (deltaStep > 5)deltaStep = 5;
			
		if (stepPredict->meanTime > 0)
		{
			step = (int)((ss_data->gyr.time - stepPredict->lastTime) / stepPredict->meanTime + 0.5);
			
			if (step > deltaStep && step <= 3)deltaStep = step;
		}
		if (lastSteps == 0)deltaStep = 0;

		if (g_pdr->heading > 0) {
			pdr_ekfStatePredict(kf, stepLength, g_pdr, deltaStep);
			g_pdr->fusionPdrFlg = PDR_TRUE;
		}
		calStepLastTime(stepPredict, ss_data->gyr.time, lastSteps, steps);
	}else{
		// ʱ򣬼Ʋܿǵȶʱ²ҪGNSSٶԤⲽﵽ
		// ߵλõĿ
		int preStep = predictStep(stepPredict, ss_data->gyr.time, lastSteps, pgnss->vel); 
		if (g_pdr->heading > 0){
			pdr_ekfStatePredict(kf, stepLength, g_pdr, preStep);
			if(preStep > 0)g_pdr->fusionPdrFlg = PDR_TRUE;
		}
	}
}

/**----------------------------------------------------------------------
* Function    : resetOutputResult
* Description : 1ʼʼλá˲ʼ
*               2GPSֵֵresult
* Date        : 2020/07/08 logzhan
*---------------------------------------------------------------------**/
int resetOutputResult(gnss *pgnss, PDR* g_pdr, KfPara *kf, lct_fs *result) {

	double stepLength = 0.7;
	double ned[3] = { 0 };

	g_pdr->pllaInit[0] = pgnss->lat;
	g_pdr->pllaInit[1] = pgnss->lon;
	g_pdr->pllaInit[2] = 0;
	wgs842Ned(g_pdr->pllaInit, g_pdr->pllaInit, ned);

	g_pdr->x0 = ned[0];
	g_pdr->y0 = ned[1];

	kf->p_xk[0] = ned[0];
	kf->p_xk[1] = ned[1];
	kf->p_xk[2] = stepLength;
	// GPSʱGPSĺǲһ׼
	kf->p_xk[3] = pgnss->yaw*RADIAN_PER_DEG;

	for (uchar i = 0; i < N; i++) {
		kf->xk[i] = kf->p_xk[i];
	}

	for (int i = 0; i < N; i++) {
		for (int l = 0; l < N; l++) {
			kf->q[i][l] = 0.0;
			kf->r[i][l] = 0.0;
		}
	}
	//printf("PDR˲GPSλ\n");
	// GPSλý
	pdr_outputGpsPos(pgnss, result);
	g_pdr->noGpsCount = 0;
	return PDR_TRUE;
}

int pdr_initProc(gnss *pgnss, KfPara *kf, PDR* g_pdr,lct_fs *result) 
{
	double stepLen = 0.7;      // ʼ˶Ϊ0.7m
	if (pgnss->satellites_num > 4 && pgnss->HDOP < 2.0  && pgnss->vel >= 1.5  && pgnss->yaw != -1) {
		// plla λԭ(λǾγ)

		g_pdr->pllaInit[0] = pgnss->lat;
		g_pdr->pllaInit[1] = pgnss->lon;
		g_pdr->pllaInit[2] = 0;

		// plla ת ned
		double ned[3] = { 0 };
		wgs842Ned(g_pdr->pllaInit, g_pdr->pllaInit, ned);

		// NEDϵļ, λ
		g_pdr->x0 = ned[0];
		g_pdr->y0 = ned[1];

		// ʼ˲Ĺ۲
		kf->p_xk[0] = ned[0];                        // ״̬1: x
		kf->p_xk[1] = ned[1];                        // ״̬2y
		kf->p_xk[2] = stepLen;                       // ״̬3
		kf->p_xk[3] = pgnss->yaw * RADIAN_PER_DEG;   // ״̬4

		for (uchar i = 0; i < N; i++) {
			kf->xk[i] = kf->p_xk[i];
		}

		kf->initHeading = kf->p_xk[3];
		for (int i = 0; i < N; i++) {
			for (int l = 0; l < N; l++) {
				kf->q[i][l] = 0.0;
				kf->r[i][l] = 0.0;
			}
		}
		pdr_outputGpsPos(pgnss, result);
		g_pdr->sysStatus = IS_NORMAL;

		return PDR_TRUE;
	}

	if (pgnss->satellites_num > 4 && pgnss->HDOP < 2.0 && pgnss->vel > 0 && pgnss->yaw != -1) {
		pdr_outputGpsPos(pgnss, result);
		return PDR_TRUE;
	}
	return PDR_FALSE;
}

/**----------------------------------------------------------------------
* Function    : calEkfQRMatrix
* Description : GPSź˲qǹ
*               ҪߵԤλþءrΪGNSS۲GPS
*               Ϣйء
* Date        : 2020/07/09 logzhan
*---------------------------------------------------------------------**/
void calEkfQRMatrix(lct_nmea *nmea, PDR *g_pdr, classifer *sys,
	KfPara *kf) {

	if (g_pdr->motionType != PDR_MOTION_TYPE_HANDHELD && 
		g_pdr->motionType != PDR_MOTION_TYPE_STATIC) {
		kf->q[0][0] = 25;                   // PDRԤλ
		kf->q[1][1] = 25;                   // PDRԤλ
		kf->q[2][2] = 1;                     // PDR
		kf->q[3][3] = 100;                  // PDR

		kf->r[0][0] = 50;                    // GPS۲λ
		kf->r[1][1] = 50;					 // GPS۲λ
		kf->r[2][2] = 0.1;					 // GPS
		kf->r[3][3] = 50;					 // GPS
		return;
	}

	kf->q[0][0] = 0.1;                       // PDRԤλ
	kf->q[1][1] = 0.1;                       // PDRԤλ
	kf->q[2][2] = 1;                         // PDR
	kf->q[3][3] = 1;                         // PDR
										     
	kf->r[0][0] = 100;                       // GPSԤλ
	kf->r[1][1] = 100;					     // GPSԤλ
	kf->r[2][2] = 0.1;					     // GPS
	kf->r[3][3] = 1000;				         // GPS

	if (nmea->accuracy.err > 0 && 
		nmea->accuracy.err < 3.80 && 
		nmea->gga.hdop <= 0.4) 
	{
		kf->r[0][0] = 50;                    // GPS۲λ
		kf->r[1][1] = 50;					 // GPS۲λ
		kf->r[2][2] = 0.1;					 // GPS
		kf->r[3][3] = 50;					 // GPS
	}

	if (fabs(R2D((g_pdr->gpsHeading - g_pdr->lastGpsHeading))) > 50 ||
		g_pdr->gpsSpeed < 0.7 || g_pdr->gpsHeading == -1) {
		kf->r[3][3] = 10000;
	}
}
/**----------------------------------------------------------------------
* Function    : pdr_gnssInsLocFusion
* Description : PDRGNSSINSں϶λ
* Date        : 2020/07/09 logzhan
*---------------------------------------------------------------------**/
void pdr_gnssInsLocFusion(gnss *pgnss, PDR *g_pdr, classifer *sys, double yaw_bias,
	KfPara *kf, lct_fs *res) {
	double plla[3] = { 0 };
	double ned[3] = { 0 };
	double yaw_thr = 6;

	int scene_type = g_pdr->sceneType;

	plla[0] = pgnss->lat;
	plla[1] = pgnss->lon;
	plla[2] = 0;

	wgs842Ned(plla, g_pdr->pllaInit, ned);

	pgnss->xNed = ned[0];
	pgnss->yNed = ned[1];

	//ںϲ
	pdr_ekfStateUpdate(kf, pgnss, sys, g_pdr, scene_type);

	for (uchar i = 0; i < N; i++) {
		for (uchar l = 0; l < N; l++) {
			kf->p_pk[i][l] = kf->pk[i][l];
		}
		kf->p_xk[i] = kf->xk[i];
	}

	ned[0] = kf -> p_xk[0] - g_pdr->x0;
	ned[1] = kf -> p_xk[1] - g_pdr->y0;
	ned[2] = 0;

	ned2Wgs84(g_pdr->pllaInit, ned, plla);

	res->latitude        = plla[0];
	res->latitude_ns     = pgnss->lat_ns;
	res->longitudinal    = plla[1];
	res->longitudinal_ew = pgnss->lon_ew;
	res->time            = pgnss->timestamps;

	g_pdr->noGpsCount = 0;
}
