/******************** (C) COPYRIGHT 2020 Geek************************************
* File Name          : pdr_api.c
* Department         : Sensor Algorithm Team
* Current Version    : V2.0
* Author             : 
* 
* Date of Issued     : 2020.7.3
* Comments           : PDR㷨ⲿӿ
********************************************************************************/
/* Header File Including -----------------------------------------------------*/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "pdr_api.h"
#include "pdr_base.h"
#include "steps.h"
#include "pdr_location.h"
#include "parseData.h"
#include "scene_recognition.h"
#include "pdr_trackSmooth.h"
#include "LapProcess.h"

/* Global Variable Definition ------------------------------------------------*/
const char* PDR_Version                     = "1.0";
size_t      slideWindowSize                 = 5; 
sensor_agm  SensorDataHist[IMU_LAST_COUNT]  = {{0}};
LatLng      llaLast                         = { 0 };
lct_nmea    NmeaData;
imu         ImuData;

/* Extern Variable Definition ------------------------------------------------*/
extern KfPara_t   g_kfPara;
extern AHRS_t     g_ahrs;
extern GNSS_t     pgnss;

/* Function Declaration ------------------------------------------------------*/

/**---------------------------------------------------------------------
* Function    : RMC_Init
* Description : ʼRMCЭ
* Date        : 2022/09/16 logzhan
*---------------------------------------------------------------------**/
void RMC_Init(lct_nmea_rmc * rmc)
{
	rmc->type            = SENSOR_MIN;
	rmc->rmc_utc         = ITEM_INVALID;
	rmc->status          = POSITIONING_N;
	rmc->latitude        = ITEM_INVALID;
	rmc->rmc_latitude_ns = LATITUDE_N;
	rmc->longitudinal    = ITEM_INVALID;
	rmc->longitudinal_ew = LATITUDE_N;
	rmc->speed           = ITEM_INVALID;
	rmc->cog             = ITEM_INVALID;
	rmc->utc_ddmmyy      = ITEM_INVALID;
	rmc->magnetic        = ITEM_INVALID;
	rmc->mag_h           = MAG_E;
	rmc->mode            = MODE_A;
	rmc->time            = ITEM_INVALID;
	rmc->update          = UN_UPDATE;

	memset(rmc->rmc_check_sum, 0, sizeof(rmc->rmc_check_sum));
}

/**---------------------------------------------------------------------
* Function    : NmeaFlag_Init
* Description : ʼGPSNMEAЭر־λ
* Date        : 2020/7/3 logzhan
*---------------------------------------------------------------------**/
void NmeaFlag_Init(lct_nmea * ln)
{
	ln->gga.update = 0;
	for (char i = 0; i < 3; i++) {
		ln->gsa[i].update = UN_UPDATE;
		ln->gsv[i].update = UN_UPDATE;
	}

	ln->minTime = 0;
	ln->maxTime = 0;
	ln->update  = UN_UPDATE;

	RMC_Init(&ln->rmc[0]);
	RMC_Init(&ln->rmc[1]);

	ln->accuracy.time   = ITEM_INVALID;
	ln->accuracy.update = UN_UPDATE;
	ln->accuracy.status = ITEM_INVALID;
	ln->accuracy.err    = ITEM_INVALID;
}


/**---------------------------------------------------------------------
* Function    : clearNmeaFlg
* Description : Nmea־λ
* Date        : 2020/7/4 logzhan
*---------------------------------------------------------------------**/
void ClearNmeaFlg(lct_nmea * ln){
    NmeaFlag_Init(ln);
}

/**---------------------------------------------------------------------
* Function    : Algorithm_Init
* Description : PDR㷨ʼ̣PDRʼƲ״̬ʼԼ
*               ƽڵĳʼ
* Date        : 2020/7/3 logzhan
*---------------------------------------------------------------------**/
void Algorithm_Init(void) {
    // سʼ
    NavSys_Init();
    // Ʋʼ
    Pedometer_Init();
    // ù켣ƽڴС
    TrackSmoothSetWinSize(slideWindowSize);
	// ȦŻʼ
	lapInit();
    // ʼsensorʷ
	memset(SensorDataHist, 0, sizeof(SensorDataHist));
	memset(&NmeaData, 0, sizeof(NmeaData));
	memset(&ImuData, 0, sizeof(ImuData));
}

/**---------------------------------------------------------------------
* Function    : ParseLineAndUpdate
* Description : ַϢ
* Date        : 2020/07/03 logzhan
*               2020/02/02 logzhan : ӺǷƽ
*---------------------------------------------------------------------**/
int ParseLineAndUpdate(char* line, lct_fs* locFusion) {

	ParseLineStr(line, &ImuData, &NmeaData);
	return LocationMainLoop(&ImuData, &NmeaData, locFusion, NULL);
}


/**---------------------------------------------------------------------
* Function    : LocationMainLoop
* Description : PDRλѭҪϲĴݲλ
*               ƽڵĳʼ
* Date        : 2020/07/03 logzhan
*               2020/02/02 logzhan : ӺǷƽ
*---------------------------------------------------------------------**/
int LocationMainLoop(imu *ImuData, lct_nmea *NmeaData, lct_fs *LocFusion, FILE *fp_gps) {
    int type = 0;

    if (ImuData->gyr.update) {
        // ǸùԴ
        InsLocation(ImuData,&g_kfPara);
        ImuData->gyr.update = UN_UPDATE;
    }
    // GPS£
    if (!NmeaData->update)return TYPE_FIX_NONE;

	// дGPSLOGϢ
	SaveGnssInfo(NmeaData, LocFusion, fp_gps);

    // GPSGPSں϶λ
    int flag = GnssInsFusionLocation(NmeaData, &g_kfPara, LocFusion);

    if (flag != TYPE_FIX_NONE) {
		// ǿֱ֣GPSƽ
        if (flag == TYPE_FIX_GPS) {
            LocFusion->latitude = R2D(LocFusion->latitude);
            LocFusion->longitudinal = R2D(LocFusion->longitudinal);

			lapProcess(pgnss, LocFusion);
        } else {
			LatLng lla = { 0 };
            lla.lat = R2D(LocFusion->latitude);
            lla.lon = R2D(LocFusion->longitudinal);

            if (LocFusion->last_lat != 0.0 && LocFusion->last_lon != 0.0 &&
                CalDistance(lla, llaLast) > 10) {
                TrackSmoothSetWinSize(slideWindowSize);
            }
			LocFusion->latitude = R2D(LocFusion->latitude);
			LocFusion->longitudinal = R2D(LocFusion->longitudinal);
			lapProcess(pgnss, LocFusion);
			LocFusion->latitude = (LocFusion->latitude) * RADIAN_PER_DEG;
			LocFusion->longitudinal = (LocFusion->longitudinal) * RADIAN_PER_DEG;

#if			PDR_OUTPUT_SMOOTH
			double outLat = 0, outLon = 0;
            // 켣ƽ
            pdr_trackSmooth(R2D(result->latitude), R2D(result->longitudinal) ,
                           &outLat, &outLon);

			result->latitude     = outLat;
			result->longitudinal = outLon;
#else
			LocFusion->latitude = R2D(LocFusion->latitude);
			LocFusion->longitudinal = R2D(LocFusion->longitudinal);
#endif
            LocFusion->last_lat = LocFusion->latitude;
			LocFusion->last_lon = LocFusion->longitudinal;

            llaLast.lat = lla.lat;
            llaLast.lon = lla.lon;
        }
        type = 1;
    }else if (LocFusion->last_lat != 0.0 && LocFusion->last_lon != 0.0) {
        LocFusion->latitude = LocFusion->last_lat;
        LocFusion->longitudinal = LocFusion->last_lon;
        type = 1;
		lapProcess(pgnss, LocFusion);
    }
    ClearNmeaFlg(NmeaData);
    return type;
}

/**---------------------------------------------------------------------
* Function    : pdr_saveGnssInfo
* Description : GPSϢ
* Date        : 2020/7/3 logzhan
*---------------------------------------------------------------------**/
void SaveGnssInfo(lct_nmea* nmea_data, lct_fs* result, FILE* fp_gps)
{
	lct_nmea_rmc rmc = nmea_data->rmc[1];
	// ѡȡʱrmc
	if (nmea_data->rmc[0].time > nmea_data->rmc[1].time) {
		rmc = nmea_data->rmc[0];
	}
	if (rmc.update && (rmc.status == POSITIONING_Y)) {
		char str[256];
		memset(str, 0, sizeof(str));
		float accuracy = -1.0f;
		if (fabs(rmc.time - nmea_data->accuracy.time) < 500){
			accuracy = nmea_data->accuracy.err;
		}
		sprintf(str, "%f,%.10f,%d,%.10f,%d,%.10f,%f,%f", rmc.time, rmc.latitude, 
			rmc.rmc_latitude_ns,rmc.longitudinal, rmc.longitudinal_ew, rmc.cog, accuracy, 
			rmc.speed);

		WriteCsvStr(fp_gps, str);

		result->gpsLat = rmc.latitude;
		result->gpsLon = rmc.longitudinal;
	}
}

/**----------------------------------------------------------------------
* Function    : GetLIBVersion
* Description : ȡpdr汾
* Date        : 2020/8/3 logzhan
*---------------------------------------------------------------------**/
const char* GetLIBVersion(void){
    return PDR_Version;
}

/**----------------------------------------------------------------------
* Function    : pdr_closeAlgorithm
* Description : رPDR㷨,ͷԴ
* Date        : 2020/8/3 logzhan
*---------------------------------------------------------------------**/
void Algorithm_DeInit() {
	freeTrackSmooth();
	scene_recognition_destroy();
}