/******************** (C) COPYRIGHT 2020 Geek************************************
* File Name          : pdr_location.c
* Current Version    : V1.2
* Author             : logzhan
* Date of Issued     : 2020.7.3
* Comments           : PDR㷨
********************************************************************************/
/* Header File Including -----------------------------------------------------*/
#include <stdio.h>
#include <math.h> 
#include <string.h>
#include <malloc.h>
#include <errno.h>
#include <stdlib.h>
#include <stdbool.h>
#include "pdr_location.h"
#include "pdr_kalman.h"
#include "steps.h"
#include "pdr_ahrs.h"
#include "pdr_detector.h"
#include "pdr_util.h"
#include "scene_recognition.h"

/* Global Variable Definition ------------------------------------------------*/
PDR*            pdr;                                  // PDRȫϢ
StepPredict     stepPredict = { 0 };
classifer       sys;
GNSS_t            pgnss;                                // λϢ
KfPara_t          g_kfPara;
double          GpsPos[HIST_GPS_NUM][3] = {{0}};
static uint32_t g_status;
extern AHRS_t     g_ahrs;

double refGpsYaw    = -1000;
double yaw_bias     = -1;

float dir[2] = { 0 };
float bias_dir[2] = { 0 };

/**---------------------------------------------------------------------
* Function    : pdr_navSysInit
* Description : PDRϵͳʼ
* Date        : 2020/2/1 logzhan
*---------------------------------------------------------------------**/
void NavSys_Init(void) {

	memset(&g_kfPara,    0, sizeof(g_kfPara));
	memset(&pgnss,       0, sizeof(pgnss));
	memset(GpsPos,       0, sizeof(GpsPos));
	memset(&stepPredict, 0, sizeof(stepPredict));
	memset(&sys,         0, sizeof(sys));
	memset(dir,          0, sizeof(dir));
	memset(bias_dir,     0, sizeof(bias_dir));

	// GNSSλʼ
    InitGnssInfo();
    // ʼAHRS̬
    AHRS_Init();
    // ʼ
    Detector_Init();
    // PDRʼ
    pdr = Base_Init();
	//baseļ̬Ǽʼ
	KalmanFilter_Init();
    // ʶģʼ
    initSceneRecognition();

	yaw_bias = -1;
}

/**---------------------------------------------------------------------
* Function    : InitGnssInfo
* Description : PDS GNSSλϢʼ
* Date        : 2020/02/01 logzhan:
*               ʼ汾
*               2020/02/08: logzhan ޸initGpsΪinitGnssInfo
*               ӷϺ
*---------------------------------------------------------------------**/
void InitGnssInfo(void) {
    pgnss.lastError    = ACCURACY_ERR_MAX;
    pgnss.overVelCount = 0;
}

/**---------------------------------------------------------------------
* Function    : ResetSystemStatus
* Description : PDR ϵͳ״̬
* Date        : 2022/09/16 logzhan
*---------------------------------------------------------------------**/
void ResetSystemStatus(KfPara_t *kf) 
{
    memset(kf, 0, sizeof(KfPara_t));
    pdr->fusionPdrFlg = PDR_FALSE;
    pdr->sysStatus    = IS_INITIAL;
    pgnss.overVelCount = 0;
}

/**---------------------------------------------------------------------
* Function    : InsLocation
* Description : PDR Եλ
* Date        : 2022/09/16 logzhan
*---------------------------------------------------------------------**/
int InsLocation(imu* ss_data, KfPara_t* kf) {

    // AHRS̬ռ2%ʱ
    if (UpdateAHRS(ss_data)) {
        // ̬ƺPCA ռ26%ʱ
        pdr_computePCA(&g_ahrs);
    }
    // û˶ͼIMUϢԼû˶ͼ, ռ21%ʱ
    updateDetectorImu(ss_data);

    pdr->gyroTime = ss_data->gyr.time;
    // Ϊʲôֳ0.1˶Ƶʣ
    if (pdr->motionType == DETECTOR_TYPE_HANDHELD)pdr->motionFreq = 0.1f;

    // Ʋ봫ݣռ10%ʱ
    updatePedometer(ss_data, &pdr->steps);

    // ----- Filter Initialization ------- //
    if (pdr->sysStatus == IS_INITIAL) {
        pdr->ts = ss_data->gyr.time;
        pdr->lastSteps = pdr->steps;
        stepPredict.lastTime = 0;
        stepPredict.fnum = 0;
        stepPredict.fsum = 0;
        return 0;
    }

    pdr->ts = ss_data->gyr.time;

    if ((0 == ss_data->acc.s[0] && 0 == ss_data->acc.s[1] && 0 == ss_data->acc.s[2]) ||
        (0 == ss_data->mag.s[0] && 0 == ss_data->mag.s[1] && 0 == ss_data->mag.s[2])) {
        return 0;
    }

    // ʶǰ״̬Ƿƽȣϵͳ10%ļ, ԺŻ
    // stateRecognition(ss_data->acc.s, &sys);

    pdr->heading = calPredAngle(pdr, &sys, pdr->gpsHeading, pdr->trackHeading, ss_data);
    if (yaw_bias != -1) {
        if (pdr->pca_accuracy > 0 && pdr->pca_accuracy < 0.1 && pdr->motionType == 3 &&
            (pdr->bias_accuracy > 1.5 || pdr->bias_accuracy == -1 || pgnss.error > 30)
            && (fabs(yaw_bias - 2 * PI) < YAW_THRES * PI / 180 || fabs(yaw_bias - PI) < YAW_THRES * PI / 180 ||
                fabs(yaw_bias) < YAW_THRES * PI / 180)) {
            dir[0] = pdr->pca_direction[0] * bias_dir[0] + pdr->pca_direction[1] * bias_dir[1];
            dir[1] = pdr->pca_direction[1] * bias_dir[0] - pdr->pca_direction[0] * bias_dir[1];
            pdr->heading = atan2(dir[1], dir[0]);
            if (pdr->heading < 0) {
                pdr->heading += TWO_PI;
            }
        }
    }

    double imuYaw = - g_ahrs.yaw;
    if (imuYaw < 0) {
        imuYaw += 360;
    }

    pdr->heading = imuYaw / (180 / 3.14159265);
    if (fabs(pdr->insHeadingOffset) > 0.000001) {
        imuYaw = imuYaw - pdr->insHeadingOffset * (180 / 3.14159265);
        pdr->heading = imuYaw / (180 / 3.14159265);
    }

    // PDR λԤ
    pdr_insPredict(pdr, &stepPredict, ss_data, &pgnss, kf);

    pdr->lastSteps   = pdr->steps;
    pdr->lastHeading = pdr->heading;
    return 0;
}

/**----------------------------------------------------------------------
* Function    : pdr_gnssInsLocation
* Description : PDR GPSںINSԶλ
* Date        : 2021/01/29 logzhan 
*---------------------------------------------------------------------**/
int GnssInsFusionLocation(lct_nmea *nmea_data, KfPara_t *kf, lct_fs *result)
{
    // GPSݸ
    GnssUpdate(&pgnss, nmea_data);
    // ʱǷ
    if (fabs(pgnss.timestamps - pdr->ts) >= 1000)return TYPE_FIX_NONE;

	// nmeaʶ𳡾Ƿǿճ
	pdr->sceneType = isOpenArea(nmea_data);
    // HDOPٶȡжgps yawǷ
   
    pdr->gpsSpeed = pgnss.vel * GPS_SPEED_UNIT;
    pdr->lastGpsHeading = pdr->gpsHeading;
    pdr->gpsHeading = GnssCalHeading(&pgnss);
    
    // win32ƽ̨·,
#ifdef _WIN32
    result->pdrHeading = kf->p_xk[3];
    result->gpsHeading = pdr->gpsHeading;
    result->hdop       = pgnss.HDOP;
    result->accuracy   = nmea_data->accuracy.err;
    result->gpsSpeed   = pdr->gpsSpeed;
    result->motionType = pdr->motionType;
#endif

    calPdrHeadingOffset(nmea_data, pdr);
    
    // GPSʱ֮䣬仯
    pdr->deltaStepsPerGps = pdr->steps - pdr->lastGpsSteps;
    pdr->lastGpsSteps = pdr->steps;

    int mode = pdr_detectFixMode(&pgnss, kf, pdr, result);
    if (mode != TYPE_FIX_PDR)return mode;

	//ʷGPSԤGPS켣
    pdr->trackHeading = calGnssTrackHeading(GpsPos, pgnss);
	// ϵͳûгʼȳʼ
    if (pdr->sysStatus == IS_INITIAL) {
        return pdr_initProc(&pgnss, kf, pdr, result);
    }
    // ߵûдﵽԤ켣
    if (!pdr->fusionPdrFlg)return resetOutputResult(&pgnss, pdr, kf, result);

    // GPSϢ, kfĹ͹۲
    CalEKFQRMatrix(nmea_data, pdr, &sys, kf);

    if (pgnss.satellites_num > 4){
        // GPSǴЧ״̬ٶҲЧ״̬
        if (pgnss.yaw == INVAILD_GPS_YAW || pgnss.vel <= 0.0) {
            //if(pdr->heading != INVAILD_GPS_YAW){
            //    pgnss.yaw = (float)(R2D(pdr->heading));
            //}else if (pdr->trackHeading != INVAILD_GPS_YAW) {
            //    pgnss.yaw = (float)(R2D(pdr->trackHeading));
            //}
            pgnss.yaw = (float)kf->p_xk[3];
        }
        if (pgnss.yaw != INVAILD_GPS_YAW) {
            pdr_gnssInsLocFusion(&pgnss, pdr, &sys, yaw_bias, kf, result);
            return TYPE_FIX_PDR;
        }
    }
    
    // ֱԭʼGPS
    if (pgnss.satellites_num > 4 && nmea_data->accuracy.err > 0 &&
        nmea_data->accuracy.err < 15) {
        OutputOriginGnssPos(&pgnss, result);
        pdr->NoGnssCount = 0;
        return TYPE_FIX_GPS;
    }

    // ûGPSźţôPDRҲλ, 10
    if (pdr->NoGnssCount < MAX_NO_GPS_PREDICT) {
        // GPS״̬λ
        NoGnssInfoPredict(kf, result, pdr);
        return TYPE_FIX_PDR;
    }
    pdr->NoGnssCount++;
	return 0;
}


/**----------------------------------------------------------------------
* Function    : NoGnssInfoPredict
* Description : ûgpsϢʱԤGPSλ
* Date        : 2022/09/16 
*---------------------------------------------------------------------**/
void NoGnssInfoPredict(KfPara_t* kf, lct_fs* result, PDR* p_pdr)
{
    double ned[3] = {0}, lla[3] = {0};
    ned[0] = kf->p_xk[0] - p_pdr->x0;
    ned[1] = kf->p_xk[1] - p_pdr->y0;
    ned[2] = 0;

    ned2Wgs84(p_pdr->pllaInit, ned, lla);

    result->latitude        = lla[0];
    result->latitude_ns     = pgnss.lat_ns;
    result->longitudinal    = lla[1];
    result->longitudinal_ew = pgnss.lon_ew;
    result->time            = pgnss.timestamps;
    p_pdr->NoGnssCount++;
}

/**----------------------------------------------------------------------
* Function    : calPdrHeadingOffset
* Description : GPSźŽϺʱƫƼ㷽ƫ
* Date        : 2020/07/08 logzhan
*---------------------------------------------------------------------**/
void calPdrHeadingOffset(lct_nmea* nmea_data, PDR* p_pdr) {
    //ߵ̬GPSнǶȵĶ׼жGPSźǷԼߵǷƽȣ
    if (refGpsYaw != -1000)return;

    if ((pgnss.satellites_num > 4 && pgnss.HDOP < 2.0 && pgnss.yaw != -1 && pgnss.vel >= 1.0)
        && (nmea_data->accuracy.err > 0 && nmea_data->accuracy.err < 4)
        && (pdr_detStatic(pdr, &pgnss, (p_pdr->steps - p_pdr->lastSteps)) == 0 && pdr->motionType == 2))
    {
        /*̬GPS׼part1 start*/
        double imuHeading = atan2(2 * (g_ahrs.qnb[0] * g_ahrs.qnb[3] + g_ahrs.qnb[1] * g_ahrs.qnb[2]), 
            1 - 2 * (g_ahrs.qnb[2] * g_ahrs.qnb[2] + g_ahrs.qnb[3] * g_ahrs.qnb[3]));
        if (imuHeading < 0) {
            imuHeading += TWO_PI;
        }

        if (fabs(imuHeading - p_pdr->gpsHeading) < PI / 18)
        {
            //printf("offset 궨\n");
            //p_pdr->insHeadingOffset = imuHeading - p_pdr->gpsHeading;
        }
    }
}

/**----------------------------------------------------------------------
* Function    : pdr_detectFixMode
* Description : ⵱ǰPDRڵģʽǳغ;ֹģʽ,ѡ
*               ԭʼGPS߲
* Date        : 2020/07/08 logzhan
*               2020/02/08 logzhan : ޸-1ΪINVAILD_GPS_YAW
*               Ŀɶ
*---------------------------------------------------------------------**/
int pdr_detectFixMode(GNSS_t* pgnss, KfPara_t* kf, PDR* g_pdr, lct_fs* result) {
    // ûǷھֹ״̬
    if (pdr_detStatic(pdr, pgnss, g_pdr->deltaStepsPerGps)) {
        if (pgnss->error > 0 && pgnss->error < 15 &&
            pgnss->lastError >= pgnss->error) {
            OutputOriginGnssPos(pgnss, result);
            pgnss->lastError = pgnss->error;
            return TYPE_FIX_PDR;
        }
        // λ
        return TYPE_FIX_NONE;
    }
    // Ǽ⵽ǿ״̬GPSλ
    if (DetectCarMode(pgnss, pdr, pdr->deltaStepsPerGps, &pgnss->overVelCount)) {
        // ⵽ģʽPDR״̬Ҫ³ʼ
        ResetSystemStatus(kf);
        if (pgnss->error >= 0 || (pgnss->satellites_num > 4
            && pgnss->HDOP < 2.0)) {
            OutputOriginGnssPos(pgnss, result);
            return TYPE_FIX_GPS;
        }
        // ȹλ
        return TYPE_FIX_NONE;
    }
    return TYPE_FIX_PDR;
}


/**----------------------------------------------------------------------
* Function    : GnssCalHeading
* Description : ȡGPSƫ(0 - 360)
* Date        : 2022/09/16 logzhan
*               Ŀɶ
*---------------------------------------------------------------------**/
double GnssCalHeading(GNSS_t* pgnss) {
    double gnssHeading = INVAILD_GPS_YAW;
    if (pgnss->satellites_num > 4 && pgnss->HDOP < 2.0 && pgnss->vel >= 1.0
        && pgnss->yaw != INVAILD_GPS_YAW && pgnss->error < 50) {
        gnssHeading = pgnss->yaw * RADIAN_PER_DEG;
    }
    return gnssHeading;
}

/**----------------------------------------------------------------------
* Function    : pdr_nmea2Gnss
* Description : nmeaݽṹתgnss
* Date        : 2020/07/08 logzhan
*---------------------------------------------------------------------**/
void Nmea2Gnss(lct_nmea* nmea_data, GNSS_t* pgnss) 
{
    lct_nmea_rmc rmc = { 0 };
    if (nmea_data->rmc[0].time > nmea_data->rmc[1].time) {
        rmc = nmea_data->rmc[0];
    }else {
        rmc = nmea_data->rmc[1];
    }
    // һεAccuracy Error
    pgnss->lastError      = pgnss->error;
    pgnss->lat            = rmc.latitude * PI / 180;
    pgnss->lat_ns         = rmc.rmc_latitude_ns;
    pgnss->lon            = rmc.longitudinal * PI / 180;
    pgnss->lon_ew         = rmc.longitudinal_ew;
    //pgnss->vel            = rmc.speed * GPS_SPEED_UNIT;
    pgnss->vel            = rmc.speed;
    pgnss->HDOP           = nmea_data->gsa[0].hdop;
    pgnss->yaw            = rmc.cog;
    pgnss->quality        = 0;
    pgnss->error          = nmea_data->accuracy.err;
    pgnss->timestamps     = rmc.time;
    pgnss->satellites_num = nmea_data->gga.satellites_nb;

}
/**----------------------------------------------------------------------
* Function    : GnssUpdate
* Description : nmeaݽṹתgnss
* Date        : 2020/07/08 logzhan
*               2020/02/09 logzhan : Ӧԭpdr_gpsUpdate
*               ÷ÿGPS¿gnssṹnmeaṹʵû
*               ʲôã
*---------------------------------------------------------------------**/
void GnssUpdate(GNSS_t* gps, lct_nmea* nmea) {
    // nmeaתgnssݽṹ
    Nmea2Gnss(nmea, &pgnss);
    // GPSص£ʱ
    gpsUpdateCb(gps);
}
