/******************** (C) COPYRIGHT 2021 Geek************************************
* File Name          : pdr_base.h
* Department         : Sensor Algorithm Team
* Current Version    : V2.0
* Author             : yuanlin@vivocom & 
* 
* 
* Date of Issued     : 2021.02.01
* Comments           : PDR 㷨ͷļ
********************************************************************************/
#ifndef _PDR_BASE_H_
#define _PDR_BASE_H_
/* Header File Including -----------------------------------------------------*/
#include "stdint.h"
#include "pdr_sensor.h"

/* PDR SYS CFG ---------------------------------------------------------------*/
#define PDR_OUTPUT_SMOOTH                   0                    // Ƿ켣ƽ

/* Macro Definition ----------------------------------------------------------*/
#define PCA_NUM                             200
#define ACCURACY_ERR_MAX                    1000                 // GPSaccuracyֵ,һڳʼ
#define N                                   4                    // ά
#define OPEN_SKY                            1
#define MAX_NO_GPS_PREDICT                  10		             // GPSϢ״̬λ
#define UN_UPDATE                           0                    // Ǹ״̬        
#define DATA_UPDATE                         1                    // ݸ
#define ON                                  1				     
#define OFF                                 0				     
#define PDR_TRUE                            1				     
#define PDR_FALSE                           0		

#define TYPE_FIX_NONE                       0                    // PDRΪ
#define TYPE_FIX_GPS                        2                    // PDRΪԭʼGPS
#define TYPE_FIX_PDR                        1                    // PDRΪPDRں
															     
// PDRЯʽ													  
#define UNKNOWN                             0				     
#define FORWARD_UP_RIGHT                    1                    // ۰ұۣͷϣ⣻orۣͷϣ
#define BACKWARD_UP_LEFT                    2                    // ۰ۣͷϣ⣻orұۣͷϣ
#define BACKWARD_DOWN_RIGHT                 3                    // ۰ұۣͷ£⣻orۣͷ£
#define FORWARD_DOWN_LEFT                   4                    // ۰ۣͷ£⣻orұۣͷ£
#define LEFT_UP_FORWARD                     5                    // 㶵ͷϣǰ
#define RIGHT_UP_BACKWARD                   6                    // 㶵ͷϣ
#define RIGHT_DOWN_FORWARD                  7	                 // 㶵ͷ£ǰ
#define LEFT_DOWN_BACKWARD                  8                    // 㶵ͷ£


// PDR״̬
#define PDR_NO_ERROR                            0
#define PDR_OUT_OF_MEMORY                       1
#define PDR_MOTION_TYPE_STATIC                  0
#define PDR_MOTION_TYPE_IRREGULAR               1
#define PDR_MOTION_TYPE_HANDHELD                2
#define PDR_MOTION_TYPE_SWINGING                3
#define PDR_STATUS_RESET                        0x80
#define PDR_STATUS_CACULATING_AXIS_CEOFFICIENT  0x40
#define PDR_STATUS_PCA_STABLE                   0x20
#define PDR_STATUS_BIAS_STABLE                  0x10
#define PDR_STATUS_STABLE                       (PDR_STATUS_PCA_STABLE | PDR_STATUS_BIAS_STABLE)

// س
#define WGS84_RE			                6378137				 // WGS-84峤
#define WGS84_ECCENTR2                      6.69437999014e-3     // һƫʵƽe^2(WGS-84)
#define WGS84_OMEGA_E_DOT                   7.2921151467e-5      // תٶ rad/sec
#define WGS84_GRAVITY		                9.7803267714		 // ƽٶ m/s^2

#define DEG_PER_RADIAN                      57.295779513082323   // degrees per radian
#define RADIAN_PER_DEG                      0.017453292519943    // radians per degree

#define GPS_SPEED_UNIT                      0.5144444            // GPSԭʼٶתm/sϵ
#define PI                                  3.1415926 
#define DOUBLE_ZERO                         1e-10
#define TWO_PI                              (2.0*PI)
#define R2D(x)	                            (x*57.2957795130823)
#define D2R(x)                              (x*3.14159265/180.0)
#define USE_BUG_FOR_LOCAL_PARA              1
#define BUF_DMT_1                           1
#define BUF_DMT_2                           2

#define PRINT_ALGO_RUN_TIME                 0
#define PRINT_CLCT                          1
#define PATTERN_RECOGNITION_LEN             256                  // 256 / SAMPLE_RATE

#define OPEN_AREA                           1                    // (GPSźŽϺ)
#define UN_OPEN_AREA                        0                    // ǿ(GPSźŽ)


#define DETECTOR_RUN_FREQ                   1280                 // û˶ͼms
#define DETECTOR_TYPE_STATIC                0                    // ûֹ 
#define DETECTOR_TYPE_IRREGULAR             1                    // ޹˶
#define DETECTOR_TYPE_HANDHELD              2                    // ֳ˶
#define DETECTOR_TYPE_SWINGING              3                    // ˶

#define ulong                               unsigned long
#define uchar                               unsigned char

typedef enum {
	IS_INITIAL = 0,
	IS_NORMAL = 1,
}pdrStatus;


typedef struct {
	double xk[N];                               // ״̬Xk              xk[0]: x  xk[1]y  xk[2]  xk[3] 
	double p_xk[N];                             // ԤXk_predict  xk[0]: x  xk[1]y  xk[2]  xk[3] 
	double zk[N];				                  
	double p_pk[N][N];			                  
	double pk[N][N];			                  
	double phi[N][N];			                
	double hk[N][N];			                
	double q[N][N];                             // ˲Q()
	double r[N][N];                             // ˲R(۲)
	double Kk[N][N];
	double lambda;
	double plat;
	double plon;
	double initHeading;
}KfPara_t;

typedef struct{
	int fnum;
	int deltaStep;
	float fsum;
	float meanTime;
	double lastTime;
}StepPredict;

typedef struct PDR {
	uint32_t        status;                     // PDRǰ״̬
	uint32_t        motionType;                 // û˶
	uint32_t        NoGnssCount;                // ûGNSSϢ
	pdrStatus       sysStatus;                  // PDRϵͳ״̬
	int             sceneType;                  // ͣ10ǿ(źŽ)
	int             fusionPdrFlg;               // ںPDRλñ־λ, flgΪ0ʱGPS
	double          pllaInit[3];                // ʼplla
	double          ts;                         // ʱ
	double          x0;                         // ʼ
	double          y0;                         // ʼ
	// 
	double          heading;                    // PDR
	double          lastHeading;                // һPDR
	double          deltaHeading;               // ƫǱ仯
	double          insHeadingOffset;           // ߵƫ 
	double          imuDeltaHeading;            // ûһGPSźţPDR仯ĽǶȣת            
	double          gpsHeading;                 // GPS
	double          lastGpsHeading;             // һGPS
	double          trackHeading;               // GPS켣
	// ٶ
	double          gpsSpeed;                   // GPSٶ
	// 
	ulong           steps;                      // ǰϢ
	ulong           lastSteps;                  // һεĲ
	ulong           lastGpsSteps;               // һGPS
	ulong           deltaStepsPerGps;           // GPS֮䣬ı仯
	float           motionFreq;                 // ˶Ƶ
	double          gyroTime;                   // ʱ
	float           axis_ceofficient[3];
	float           axis_direction[2];
	float           pca_direction[2];
	float           pca_accuracy;
	float           bias_direction[2];
	float           bias_accuracy;
	float           cal_direction[2];
} PDR;

typedef struct {
	int type;
	float accBuffer[PATTERN_RECOGNITION_LEN];
}classifer;

// û˶ͷ
typedef struct DETECTOR {
	uint32_t type;                              // û˶  0:ֹ˶ 1޹˶ 2ֳ˶ 3˶
	uint32_t lastType;
	uint64_t tick;                              // ͳƣڵƵ
} DETECTOR;


DETECTOR *pdr_getDetector(void);


/* Function Declaration ------------------------------------------------------*/

PDR* Base_Init(void);

/**----------------------------------------------------------------------
* Function    : pdr_resetData
* Description : PDR
* Date        : 2020/7/21 
* 
*---------------------------------------------------------------------**/
void pdr_resetData(void);

/**----------------------------------------------------------------------
* Function    : pdr_computePCA
* Description : RESET_PHASE_1\STABLE׶֮PCA
* Date        : 2020/7/21 logzhan
*---------------------------------------------------------------------**/
void ComputePCA(AHRS_t* ahrs);

/**---------------------------------------------------------------------
* Function    : detectorUpdate
* Description : ȷֻЯʽg_pdr.status
* Date        : 2020/7/20 logzhan
*---------------------------------------------------------------------**/
void detectorUpdate(DETECTOR* detector);


void gpsUpdateCb(GNSS_t* gps);


/**************************************************************************
* Description : ÿʱ
* Input       : stepPredict Ԥṹ
				timestampʱ
				steps_lastһβ
				stepsβ
**************************************************************************/
void calStepLastTime(StepPredict *stepPredict, double timestamp, unsigned long steps_last, unsigned long steps);

/**************************************************************************
* Description : Ԥⲽ
* Input       : stepPredict Ԥṹ
				timestampʱ
				steps_lastһβ
				gnssVelgpsٶ
* Output      : int 
**************************************************************************/
int predictStep(StepPredict *stepPredict, double timestamp, unsigned long steps_last, float gnssVel);

/**----------------------------------------------------------------------
* Function    : stateRecognition
* Description : ݼٶȼжϵǰ״̬Ƿƽ 1: ƽ  0: ƽ
* Date        : 2020/7/22 logzhan
*---------------------------------------------------------------------**/
void StateRecognition(float *acc, classifer *sys);

/**************************************************************************
* Description : GPSǶȡGPS켣ǡPDRǶȣû߷
* Input       : g_pdrPDRṹ
				sys˶ݽṹ
				gps_yawGPS켣
				G_yawGPS
				ss_data ݽṹ
* Output      : doubleû߷
**************************************************************************/
double calPredAngle(PDR *g_pdr, classifer *sys, double gps_yaw, double G_yaw, IMU_t *ss_data);

/**************************************************************************
* Description : GPSǶȡGPS켣ǡPDRǶȣû߷
* Input       : g_pdrPDRṹ
				stepsβ
				steps_lastһβ
				stepPredictԤṹ
				pdr_angleû߷
				scene_typeGPSûͷǿ
				ss_data ݽṹ
				pgnssGPSݽṹ
				kfEKFṹ
				pdr, PDRԤλóɹ־λ
**************************************************************************/
void  InsLocationPredict(PDR *g_pdr, StepPredict *stepPredict,IMU_t *ss_data, 
	GNSS_t *pgnss, KfPara_t *kf);

/**----------------------------------------------------------------------
* Function    : pdr_detStatic
* Description : ûǷھֹ״̬
* Date        : 2021/01/28 logzhan
*---------------------------------------------------------------------**/
int pdr_detStatic(PDR *g_pdr, GNSS_t *pgnss, unsigned long delSteps);


/**----------------------------------------------------------------------
* Function    : pdr_outputGpsPos
* Description : GPSλ
* Date        : 2020/07/08 logzhan
*---------------------------------------------------------------------**/
void OutputOriginGnssPos(GNSS_t *pgnss, lct_fs *result);

/**----------------------------------------------------------------------
* Function    : resetOutputResult
* Description : 1ʼʼλá˲ʼ
*               2GPSֵֵresult
*               pgnssGPSݽṹ
*               plla_initNEDϵԭ㾭γ
*               x_inity_initNEDϵԭne
*               kf EKFݽṹ
* Date        : 2020/07/08 logzhan
*---------------------------------------------------------------------**/
int ResetOutputResult(GNSS_t *pgnss, PDR* g_pdr, KfPara_t *kf, lct_fs *result);

/**----------------------------------------------------------------------
* Function    : detIsCarMode
* Description : ʶǷǳģʽ
* Input       : pgnssGPSݽṹ
				g_pdrPDRṹ
				delSteps ϴȲı仯
				time
* Output      : intģʽ־λ
* Date        : 2021/01/28 logzhan
*---------------------------------------------------------------------**/
int DetectCarMode(GNSS_t *pgnss, PDR *g_pdr, unsigned long delSteps, int *time);

/**----------------------------------------------------------------------
* Function    : detPdrToReset
* Description : PDRϵͳǷҪãǵPDRʧ󣬻
*               PDRGPSλá
* Date        : 2021/01/28 logzhan
*---------------------------------------------------------------------**/
int detPdrToReset(double pdr_angle, int *gpscnt, unsigned long deltsteps, PDR *g_pdr);

/**************************************************************************
* Description : ʼEKF˲
* Input       : pgnssGPSݽṹ
				plla_initNEDϵԭ㾭γ
				kf EKFݽṹ
				x_inity_initNEDϵԭne
				sys_statusλϵͳ״̬δʼѳʼ
* Output      : intʼ־λ
				resultλṹ
**************************************************************************/
int InitProc(GNSS_t *pgnss,  KfPara_t *kf, PDR* g_pdr, lct_fs *result);

/**************************************************************************
* Description : EKF˲QR
* Input       : scane_type, ûͷǿ
				nmea_dataNMEAݽṹ
				g_pdrPDRṹ
				sys˶ݽṹ
				kfEKFݽṹ
**************************************************************************/


/**************************************************************************
* Description : GPS켣
* Input       : gpsPosGPSʷλECEFµά
				pgnssGPSݽṹ
* Output      : double켣
**************************************************************************/
double CalGnssTrackHeading(double gpsPos[][3], GNSS_t pgnss);


#endif

#pragma once
