/******************** (C) COPYRIGHT 2022 Geek************************************
* File Name          : Interface.c
* Current Version    : V2.0
* Author             : logzhan
* Date of Issued     : 2022.10.15
* Comments           : PDR㷨ⲿӿ
********************************************************************************/
/* Header File Including -----------------------------------------------------*/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "pdr_api.h"
#include "pdr_base.h"
#include "Pedometer.h"
#include "Location.h"
#include "parseData.h"
#include "scene_recognition.h"

/* Global Variable Definition ------------------------------------------------*/
const char* PDR_Version                     = "2.0";
Sensor_t    SensorDataHist[IMU_LAST_COUNT]  = {{0}};
LatLng_t    llaLast                         = { 0 };
Nmea_t      Nmea;
IMU_t       Imu;

/* Extern Variable Definition ------------------------------------------------*/
extern EKFPara_t  g_kfPara;
extern AHRS_t     g_ahrs;
extern GNSS_t     pgnss;

/* Function Declaration ------------------------------------------------------*/

/**---------------------------------------------------------------------
* Function    : RMC_Init
* Description : ʼRMCЭ
* Date        : 2022/09/16 logzhan
*---------------------------------------------------------------------**/
void RMC_Init(NmeaRMC_t * rmc)
{
	rmc->type            = SENSOR_MIN;
	rmc->rmc_utc         = ITEM_INVALID;
	rmc->status          = POSITIONING_N;
	rmc->latitude        = ITEM_INVALID;
	rmc->rmc_latitude_ns = LATITUDE_N;
	rmc->longitudinal    = ITEM_INVALID;
	rmc->longitudinal_ew = LATITUDE_N;
	rmc->speed           = ITEM_INVALID;
	rmc->cog             = ITEM_INVALID;
	rmc->utc_ddmmyy      = ITEM_INVALID;
	rmc->magnetic        = ITEM_INVALID;
	rmc->mag_h           = MAG_E;
	rmc->mode            = MODE_A;
	rmc->time            = ITEM_INVALID;
	rmc->update          = UN_UPDATE;

	memset(rmc->rmc_check_sum, 0, sizeof(rmc->rmc_check_sum));
}

/**---------------------------------------------------------------------
* Function    : NmeaFlag_Init
* Description : ʼGPSNMEAЭر־λ
* Date        : 2020/7/3 logzhan
*---------------------------------------------------------------------**/
void NmeaFlag_Init(Nmea_t * ln)
{
	ln->gga.update = 0;
	for (char i = 0; i < 3; i++) {
		ln->gsa[i].update = UN_UPDATE;
		ln->gsv[i].update = UN_UPDATE;
	}

	ln->minTime = 0;
	ln->maxTime = 0;
	ln->update  = UN_UPDATE;

	RMC_Init(&ln->rmc[0]);
	RMC_Init(&ln->rmc[1]);

	ln->accuracy.time   = ITEM_INVALID;
	ln->accuracy.update = UN_UPDATE;
	ln->accuracy.status = ITEM_INVALID;
	ln->accuracy.err    = ITEM_INVALID;
}

/**---------------------------------------------------------------------
* Function    : clearNmeaFlg
* Description : Nmea־λ
* Date        : 2020/7/4 logzhan
*---------------------------------------------------------------------**/
void ClearNmeaFlg(Nmea_t * ln){
    NmeaFlag_Init(ln);
}

/**---------------------------------------------------------------------
* Function    : Algorithm_Init
* Description : PDR㷨ʼ̣PDRʼƲ״̬ʼԼ
*               ƽڵĳʼ
* Date        : 2022/10/15 logzhan
*---------------------------------------------------------------------**/
void Algorithm_Init(void) {
    // سʼ
    PDR_Init();
    // Ʋʼ
    Pedometer_Init();
    // ʼsensorʷ
	memset(SensorDataHist, 0, sizeof(SensorDataHist));
	memset(&Nmea, 0, sizeof(Nmea));
	memset(&Imu, 0, sizeof(Imu));
}

/**---------------------------------------------------------------------
* Function    : ParseDataAndUpdate
* Description : ַϢ
* Date        : 2022/10/16 logzhan
*---------------------------------------------------------------------**/
int ParseDataAndUpdate(char* line, LctFs_t* LocFusion) 
{
	ParseLineStr(line, &Imu, &Nmea);
	// ImuNmeaṹ壬ںλ
	return PDRLocationMainLoop(&Imu, &Nmea, LocFusion);
}


/**---------------------------------------------------------------------
* Function    : PDRLocationMainLoop
* Description : PDRλѭҪϲĴݲλ
*               ƽڵĳʼ
* Date        : 2020/07/03 logzhan
*               2020/02/02 logzhan : ӺǷƽ
*---------------------------------------------------------------------**/
int PDRLocationMainLoop(IMU_t *imu, Nmea_t *nmea, LctFs_t *LocFusion) {
    int type = 0;

    if (imu->gyr.update) {
        // ǸùԴ
        InsLocation(imu,&g_kfPara);
        imu->gyr.update = UN_UPDATE;
    }
    // GPS£
    if (!nmea->update)return TYPE_FIX_NONE;

	// дGPSLOGϢ
	SaveGnssInfo(nmea, LocFusion, NULL);

    // GPSGPSں϶λ
    int flag = GnssInsFusionLocation(nmea, &g_kfPara, LocFusion);

    if (flag != TYPE_FIX_NONE) {
		// ǿֱ֣GPSƽ
		LocFusion->latitude = R2D(LocFusion->latitude);
		LocFusion->longitudinal = R2D(LocFusion->longitudinal);

        LocFusion->last_lat = LocFusion->latitude;
		LocFusion->last_lon = LocFusion->longitudinal;
        type = 1;
    }else if (LocFusion->last_lat != 0.0 && LocFusion->last_lon != 0.0) {
        LocFusion->latitude = LocFusion->last_lat;
        LocFusion->longitudinal = LocFusion->last_lon;
        type = 1;
    }
    ClearNmeaFlg(nmea);
    return type;
}

/**---------------------------------------------------------------------
* Function    : SaveGnssInfo
* Description : GPSϢ
* Date        : 2022/10/16 logzhan
*---------------------------------------------------------------------**/
void SaveGnssInfo(Nmea_t* nmea, LctFs_t* result, FILE* fp_gps)
{
	NmeaRMC_t rmc = nmea->rmc[1];
	// ѡȡʱrmc
	if (nmea->rmc[0].time > nmea->rmc[1].time) {
		rmc = nmea->rmc[0];
	}
	if (rmc.update && (rmc.status == POSITIONING_Y)) {
		char str[256];
		memset(str, 0, sizeof(str));
		float accuracy = -1.0f;
		if (fabs(rmc.time - nmea->accuracy.time) < 500){
			accuracy = nmea->accuracy.err;
		}
		sprintf(str, "%f,%.10f,%d,%.10f,%d,%.10f,%f,%f", rmc.time, rmc.latitude, 
			rmc.rmc_latitude_ns,rmc.longitudinal, rmc.longitudinal_ew, rmc.cog, accuracy, 
			rmc.speed);

		WriteCsvStr(fp_gps, str);

		result->gpsLat = rmc.latitude;
		result->gpsLon = rmc.longitudinal;
	}
}

/**----------------------------------------------------------------------
* Function    : GetPDRVersion
* Description : ȡpdr汾
* Date        : 2022/10/15 logzhan
*---------------------------------------------------------------------**/
const char* GetPDRVersion(void){
    return PDR_Version;
}

/**----------------------------------------------------------------------
* Function    : pdr_closeAlgorithm
* Description : رPDR㷨,ͷԴ
* Date        : 2020/8/3 logzhan
*---------------------------------------------------------------------**/
void Algorithm_DeInit() {
	scene_recognition_destroy();
}