/******************** (C) COPYRIGHT 2020 Geek************************************
* File Name          : Kalman.c
* Current Version    : V2.0
* Author             : logzhan
* Date of Issued     : 2022.10.15
* Comments           : PDRչ˲ں
********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#if !defined(LOCATION_PLATFORM_QCOM_MODEM)
#include <malloc.h>
#endif
#include "pdr_base.h"
#include "Kalman.h"
#include "Location.h"
#include "Matrix.h"
#include "Utils.h"
#include "LinearFit.h"
#include "CoordTrans.h"

// չ
static double EkfVecBuf[5][N] = { {0.0} };
// չ󻺴
static double EkfMatBuf[7][N][N] = { { {0.0} } };

extern double GpsPos[HIST_GPS_NUM][3];

extern int debugCount;


static int         hold_type;
static double      b_timestamp;
static double      b_last_time;
static double      attitude_yaw;
static Detector_t *pDetector;
static int         hold_type;
static double      BUL_duration;
static double      FUR_duration;

/**----------------------------------------------------------------------
* Function    : KalmanFilter_Init
* Description : ʼ˲
* Date        : 2022/09/16 logzhan
*---------------------------------------------------------------------**/
void EKF_Init(void)
{
	b_timestamp  = -1;
	b_last_time  = -1;
	BUL_duration = -1;
	FUR_duration = -1;
	attitude_yaw = -1;
	hold_type = 0;
}

/**----------------------------------------------------------------------
* Function    : EKFStatePredict
* Description : pdr˲״̬Ԥⷽ
* Date        : 2022/10/15 logzhan
*---------------------------------------------------------------------**/
void EKFStatePredict(EKFPara_t *ekf, double stepLen, PDR_t* pdr, int step) {

	double (*Phi)[N]   = (double(*)[N])&(EkfMatBuf[0][0][0]);
	double (*pvec1)[N] = (double(*)[N])&(EkfMatBuf[1][0][0]);
	double (*pvec2)[N] = (double(*)[N])&(EkfMatBuf[2][0][0]);

	memset(EkfMatBuf, 0, sizeof(double) * 7 * N * N);

	double deltaHeading = pdr->heading - pdr->lastHeading;
	double angle        = pdr->heading;

	//printf("deltaHeading = %f\n", R2D(deltaHeading));
	//if (fabs(deltaHeading) < D2R(10) && g_pdr->lastHeading != 0.0f) {
	//	angle = deltaHeading + kf->pXk[3];
	//}else {
	//	angle = g_pdr->heading;
	//}

	if (pdr->MotionType != PDR_MOTION_TYPE_HANDHELD) {
		deltaHeading = 0;
	}

	angle = deltaHeading + ekf->pXk[3];


	if (fabs(deltaHeading) > 5.5) {
		if (deltaHeading < 0) {
			deltaHeading = deltaHeading + D2R(360);
		}else {
			deltaHeading = deltaHeading - D2R(360);
		}
		deltaHeading = 0;
		angle = deltaHeading + ekf->pXk[3];
	}

	if(step == 0){
		ekf->pXk[3] = angle;
	}

    // xk+1 = xk + step * cos(angle)   PDRλø·
	// yk+1 = yk + step * sin(angle)   PDRλø·
	// sk+1 = sk                       һͬ
	// ak+1 = ak * (angle / ak)  ak+1 = ak

	Phi[0][0] = 1;            Phi[1][0] = 0;          Phi[2][0] = 0;  Phi[3][0] = 0;
	Phi[0][1] = 0;			  Phi[1][1] = 1;		  Phi[2][1] = 0;  Phi[3][1] = 0;
	Phi[0][2] = cos(angle);	  Phi[1][2] = sin(angle); Phi[2][2] = 1;  Phi[3][2] = 0;
	Phi[0][3] = 0;            Phi[1][3] = 0;          Phi[2][3] = 0;  Phi[3][3] = 1;


	for (uchar i = 0; i < step; i++) {
		// logzhan 21/02/06:
		// phi[3][3]ΪkfԤÿζ¸ֵ, phi[3][3]ÿζ1ڹ
		// ¹УƫǵԤǹ̶ģʵʱûôãifжϿһ̶
		// Ԥֵ£ЧʱûиоԵĺ
		if (fabs(ekf->pXk[3]) > 0.000001 &&
			fabs(angle / ekf->pXk[3]) < 3)
		{
			Phi[3][3] = angle / ekf->pXk[3];
		}

		// ״̬·
		// pXk = phi * pXk
		VecMatMultiply(ekf->pXk, Phi, ekf->pXk);

		// Э : Pk = FPFt + Q
		MatrixMultiply(Phi, ekf->pPk, pvec1);               // F*P
		MatrixTrans(Phi, pvec2);                            // Ft
		MatrixMultiply(pvec1, pvec2, pvec1);                // F*P*Ft
		MatrixAdd(pvec1, ekf->Q, ekf->pPk);                 // F*P*Ft + Q
	}
}

/**----------------------------------------------------------------------
* Function    : EKFStateUpdate
* Description : չ˲״̬·
* Date        : 2020/7/22 logzhan
*---------------------------------------------------------------------**/
void EKFStateUpdate(EKFPara_t *kf, GNSS_t *pgnss,  Classifer_t *sys, PDR_t *g_pdr, 
	int scene_type) {

	double lambda = 0;

	// NxN
	double(*H)[N]     = (double(*)[N])&(EkfMatBuf[0][0][0]);
	double(*I)[N]     = (double(*)[N])&(EkfMatBuf[1][0][0]);
	double(*pvec1)[N] = (double(*)[N])&(EkfMatBuf[2][0][0]);
	double(*Ht)[N]    = (double(*)[N])&(EkfMatBuf[3][0][0]);
	double(*pvec3)[N] = (double(*)[N])&(EkfMatBuf[4][0][0]);
	double(*pvec4)[N] = (double(*)[N])&(EkfMatBuf[5][0][0]);
	double(*pv3)[N]   = (double(*)[N])&(EkfMatBuf[6][0][0]);

	memset(EkfMatBuf, 0, sizeof(double) * 7 * N * N);
	memset(EkfVecBuf, 0, sizeof(double) * 5 * N);

	// Nx1
	double(*pvec5)    = &(EkfVecBuf[1][0]);
	double(*pvec6)    = &(EkfVecBuf[2][0]);
	double(*pv1)      = &(EkfVecBuf[3][0]);
	double(*pv2)      = &(EkfVecBuf[4][0]);
	
	// λ
	for (char i = 0; i < 4; i++) {
		H[i][i] = 1;
		I[i][i] = 1;
	}

	double* z = &(EkfVecBuf[0][0]);
	// ȡ۲
	z[0] = pgnss->xNed;                                          // λ
	z[1] = pgnss->yNed;                                          // λ   
	z[2] = 0.5;                                                  // Ĭ0.5

	// ifʵʹGPSλñ仯Ͳ仯Ʋ, ⲿִ뻹û
	// ãʵڲ
	//if (pgnss->error > 0 && pgnss->error < 3.8 && pgnss->HDOP <= 0.4 && 
	//	g_pdr->deltaStepsPerGps != 0 && g_pdr->deltaStepsPerGps < 5) {
	//	// GPSٶȺͲƵƲ
	//	//if (g_pdr->motionFreq > 2.5 && (sys->type == 1)) {
	//	//	
	//	//	//printf("z[2] = %f\n", z[2]);
	//	//}
	//	z[2] = pgnss->vel * 0.514 / (g_pdr->deltaStepsPerGps);
	//}else {
	//	z[2] = kf->pXk[2];
	//}

	z[3] = pgnss->yaw * PI / 180;                                // ۲ĺΪGPSĺ

	//  K = K = PHt / (R + H*P*H^T)
	MatrixTrans(H, Ht);                                             // Ht
	MatrixMultiply(H, kf->pPk, pvec1);                             // HP
	MatrixMultiply(pvec1, Ht, pvec3);                               // pvec3 = H*P*H^T
	MatrixAdd(pvec3, kf->R, pvec4);                                 // pvec4 = R + H*P*H^T
	MatrixInverse(pvec4, pvec1);                                    //  1 / (R + H*P*H^T)

	for (char i = 0; i < N; i++) {
		for (uchar j = 0; j < N; j++) {
			pv3[i][j] = pvec3[i][j];
		}
	}
	MatrixMultiply(kf->pPk, Ht, pvec3);                                  // PHt
	MatrixMultiply(pvec3, pvec1, kf->Kk);                                 // K = PHt / (R + H*P*H^T)
								                                  
								                                  
	VecMatMultiply(kf->pXk, H, pvec5);                                   // pvec5 = Hx'
	VectorSub(z, pvec5, pvec6);                                       // pvec6 = Z - Hx'
	modAngle(&pvec6[3], -PI, PI);
	for (char i = 0; i < N; i++) {
		pv1[i] = pvec6[i];
	}
	
	VecMatMultiply(pvec6, kf->Kk, pvec5); // pvec5 = K*( z - Hx')
	// Get the optimally updated state estimation
	VectorAdd(kf->pXk, pvec5, kf->Xk);

	// ---- prepare the matrix for updated estimate covariance ------
	// pvec1 = K*H
	MatrixMultiply(kf->Kk, H, pvec1);
	// pvec2 = (I - K*H)
	MatrixSub(I, pvec1, Ht);
	// pvec3 = (I - K*H)*P
	MatrixMultiply(Ht, kf->pPk, pvec3);
	// pvec4 = (I- K*H)^T
	MatrixTrans(Ht, pvec4);
	// pvec1 = (I - K*H)*P*(I- K*H)^T
	MatrixMultiply(pvec3, pvec4, pvec1);

	// pvec2 = K*R
	MatrixMultiply(kf->Kk, kf->R, Ht);
	// pvec3 = K^T
	MatrixTrans(kf->Kk, pvec3);
	// pvec4 = K*R*K^T
	MatrixMultiply(Ht, pvec3, pvec4);
	// Get the updated estimate covariance: P' = (I - K*H)*P*(I- K*H)^T + K*R*K^T
	MatrixAdd(pvec1, pvec4, kf->Pk);

	// pv2 = (z - Hx')*( H*P*H^T )
	VecMatMultiply(pv1, pv3, pv2);

	// inner product the pre-fit residual and the transitioned residual: lamda = (z - Hx')*( H*P*H^T )  .*  (z-Hx')
	// Calculate the difference between original differenc and the transitioned,
	// if the value is close to zero, it shows that there is a big gap in the 
	// oprimization. Thus, we believe our PDR algorithm
	for (char i = 0; i < N; i++) {
		lambda += pv2[i] * pv1[i];
	}

	kf->Lambda = lambda;
	kf->pLat = pgnss->lat;
	kf->pLon = pgnss->lon;

	if (lambda >= 200)return;

	for (char i = 0; i < N; i++) {
		kf->Xk[i] = kf->pXk[i];
		for (int j = 0; j < N; j++) {
			kf->Pk[i][j] = kf->pPk[i][j];
		}
	}
	
}

void calStepLastTime(StepPredict *stepPredict, double timestamp, unsigned long steps_last, 
	unsigned long steps)
{
	float tmpTime = 0;
	if (stepPredict->lastTime > 0 && stepPredict->deltaStep == 0 && (steps - steps_last) > 0)
	{
		tmpTime = (float)((timestamp - stepPredict->lastTime) / (steps - steps_last));
		if (tmpTime > 300 && tmpTime < 800)
		{
			if (stepPredict->meanTime > 0){
				stepPredict->meanTime = stepPredict->meanTime * 0.5f + tmpTime * 0.5f;
			}else{
				stepPredict->meanTime = tmpTime;
			}
		}
	}

	stepPredict->fnum = 0;
	stepPredict->fsum = 0;

	stepPredict->lastTime = timestamp;
	stepPredict->deltaStep = 0;

}

int predictStep(StepPredict *stepPredict, double timestamp, unsigned long steps_last, float gnssVel)
{
	int step = 0;
	float mean_time = 400;
	double dis = 0;

	if (stepPredict->meanTime > 0)
		mean_time = stepPredict->meanTime;

	if ((steps_last > 0) && (stepPredict->fnum > 0) && (timestamp - stepPredict->lastTime > mean_time * 3) && (stepPredict->lastTime > 0))
	{
		stepPredict->fsum = stepPredict->fsum / stepPredict->fnum;		
		dis = sqrt(pow(GpsPos[HIST_GPS_NUM - 1][0]- GpsPos[HIST_GPS_NUM - 2][0], 2) + pow(GpsPos[HIST_GPS_NUM - 1][1] - GpsPos[HIST_GPS_NUM - 2][1], 2)
			 + pow(GpsPos[HIST_GPS_NUM - 1][2] - GpsPos[HIST_GPS_NUM - 2][2], 2));

		if (stepPredict->fsum != 0.0 && (gnssVel > 1.0 || (dis > 1.0 && dis < 3.0)))
		{
			step = (int)((timestamp - stepPredict->lastTime) / mean_time);
			stepPredict->deltaStep += step;
		}

		stepPredict->lastTime = timestamp;
		stepPredict->fnum = 0;
		stepPredict->fsum = 0;
	}

	return step;
}

/**----------------------------------------------------------------------
* Function    : StateRecognition
* Description : ݼٶȼжϵǰ״̬Ƿƽ 1: ƽ  0: ƽ
* Date        : 2022/09/19 logzhan
*---------------------------------------------------------------------**/
void StateRecognition(float *acc, Classifer_t *sys) {

}


double CalGnssTrackHeading(double gpsPos[][3], GNSS_t pgnss)
{
	int i;
	double lla[3] = { 0.0 };
	double ned1[3] = { 0.0 };
	double temp[2][HIST_GPS_NUM] = { { 0.0 } };
	double angle[HIST_GPS_NUM - 1] = { 0.0 };

	double ned2[3] = { 0.0 };
	double maxn = 0;
	double maxe = 0;
	double minn = 0;
	double mine = 0;
	double meanx = 0;
	double meany = 0;

	float error = 0;
	float sst = 0;
	float r2 = 0;

	double range = 0;
	double yaw = 0;
	double tempYaw = 0;
	double diffAngle = 0;

	double para[3] = { 0 };
	int flg = 0;

	lla[0] = pgnss.lat;
	lla[1] = pgnss.lon;

	for (i = 0; i < HIST_GPS_NUM - 1; i++)
	{
		gpsPos[i][0] = gpsPos[i + 1][0];
		gpsPos[i][1] = gpsPos[i + 1][1];
		gpsPos[i][2] = gpsPos[i + 1][2];
	}

	if ((lla[0] > 0) && (lla[1] > 0))
	{
		//γWGS84תĵعϵECEF
		WGS842ECEF(lla, gpsPos[HIST_GPS_NUM - 1]);
	}
	else
	{
		gpsPos[HIST_GPS_NUM - 1][0] = 0;
		gpsPos[HIST_GPS_NUM - 1][1] = 0;
		gpsPos[HIST_GPS_NUM - 1][2] = 0;
		return -1;
	}
	if (pgnss.error > 40)
	{
		return -1;
	}

	for (i = 0; i < HIST_GPS_NUM - 1; i++)
	{
		if ((fabs(gpsPos[i][0]) < 1e-6) && (fabs(gpsPos[i][1]) < 1e-6) && (fabs(gpsPos[i][2]) < 1e-6))
		{
			return -1;
		}
	}
	//ĵعϵECEFתϵNED
	ECEF2NED(gpsPos[HIST_GPS_NUM - 1], lla, ned1);

	for (i = HIST_GPS_NUM - 2; i >= 0; i--) {

		ECEF2NED(gpsPos[i], lla, ned2);

		temp[0][i] = ned1[0] - ned2[0];
		temp[1][i] = ned1[1] - ned2[1];

		meanx += temp[0][i] / HIST_GPS_NUM;
		meany += temp[1][i] / HIST_GPS_NUM;

		if (maxn > temp[0][i])
		{
			maxn = temp[0][i];
		}
		if (minn < temp[0][i])
		{
			minn = temp[0][i];
		}
		if (maxe > temp[1][i])
		{
			maxe = temp[1][i];
		}
		if (mine < temp[1][i])
		{
			mine = temp[1][i];
		}

		angle[i] = atan2(temp[1][i], temp[0][i]);
		if (angle[i] < 0)
			angle[i] += TWO_PI;
	}
	flg = leastDistanceLinearFit(temp[0], temp[1], HIST_GPS_NUM, para);
	if (flg < 0)
		return -1;

	for (i = HIST_GPS_NUM - 1; i >= 0; i--) {
		error += (float)(vivopow(para[0] * temp[0][i] + para[1] * temp[1][i] + para[2], 2) / (vivopow(para[0], 2) + vivopow(para[1], 2)));
		sst += (float)(vivopow(meanx - temp[0][i], 2) + vivopow(meany - temp[1][i], 2));
	}

	if (sst > 0)
	{
		r2 = 1 - error / sst;
	}
	range = sqrt(vivopow(maxn - minn, 2) + vivopow(maxe - mine, 2));
	//printf("%f,%f,%f\n", r2, error, range);
	if (r2 < 0.95 || range < 1.5 || error > 3.0)
		return -1;

	yaw = atan(-para[0] / para[1]);
	modAngle(&yaw, 0, TWO_PI);

	tempYaw = meanAngle(angle, HIST_GPS_NUM - 1);
	modAngle(&tempYaw, 0, TWO_PI);

	diffAngle = tempYaw - yaw;
	modAngle(&diffAngle, -PI, PI);

	if (fabs(diffAngle) > PI / 2)
		yaw += PI;
	modAngle(&yaw, 0, TWO_PI);

	return yaw;
}

/**----------------------------------------------------------------------
* Function    : InsLocationPredict
* Description : PDRߵλø
* Date        : 2021/02/06 logzhan : оƲƽʱ
*               PDRǰλò
*---------------------------------------------------------------------**/
void InsLocationPredict(PDR_t *g_pdr, StepPredict *stepPredict, IMU_t *ss_data, 
	GNSS_t *pgnss, EKFPara_t *kf) {
	int step;
	long deltaStep;
	double stepLength = 0.7;

	stepPredict->fnum++;
	stepPredict->fsum += g_pdr->motionFreq;

	ulong  steps     = g_pdr->steps;
	ulong  lastSteps = g_pdr->lastSteps;
	
	if (steps - lastSteps > 0 && g_pdr->motionFreq != 0.0) {
		deltaStep = steps - lastSteps - stepPredict->deltaStep;

		// ӦǷֹƲ쳣һμƲر
		if (deltaStep > 5)deltaStep = 5;
			
		if (stepPredict->meanTime > 0)
		{
			step = (int)((ss_data->gyr.time - stepPredict->lastTime) / stepPredict->meanTime + 0.5);
			
			if (step > deltaStep && step <= 3)deltaStep = step;
		}
		if (lastSteps == 0)deltaStep = 0;

		if (g_pdr->heading > 0) {
			EKFStatePredict(kf, stepLength, g_pdr, deltaStep);
			g_pdr->fusionPdrFlg = PDR_TRUE;
		}
		calStepLastTime(stepPredict, ss_data->gyr.time, lastSteps, steps);
	}else{
		// ʱ򣬼Ʋܿǵȶʱ²ҪGNSSٶԤⲽﵽ
		// ߵλõĿ
		int preStep = predictStep(stepPredict, ss_data->gyr.time, lastSteps, pgnss->vel); 
		if (g_pdr->heading > 0){
			EKFStatePredict(kf, stepLength, g_pdr, preStep);
			if(preStep > 0)g_pdr->fusionPdrFlg = PDR_TRUE;
		}
	}
}


/**----------------------------------------------------------------------
* Function    : EKFCalQRMatrix
* Description : GPSź˲qǹ
*               ҪߵԤλþءrΪGNSS۲GPS
*               Ϣйء
* Date        : 2022/09/19 logzhan
*---------------------------------------------------------------------**/
void EKFCalQRMatrix(Nmea_t *nmea, PDR_t *g_pdr, Classifer_t *sys,
	EKFPara_t *kf) {

	if (g_pdr->MotionType != PDR_MOTION_TYPE_HANDHELD && 
		g_pdr->MotionType != PDR_MOTION_TYPE_STATIC) {
		kf->Q[0][0] = 25;                   // PDRԤλ
		kf->Q[1][1] = 25;                   // PDRԤλ
		kf->Q[2][2] = 1;                    // PDR
		kf->Q[3][3] = 100;                  // PDR

		kf->R[0][0] = 50;                    // GPS۲λ
		kf->R[1][1] = 50;					 // GPS۲λ
		kf->R[2][2] = 0.1;					 // GPS
		kf->R[3][3] = 50;					 // GPS
		return;
	}

	kf->Q[0][0] = 0.1;                       // PDRԤλ
	kf->Q[1][1] = 0.1;                       // PDRԤλ
	kf->Q[2][2] = 1;                         // PDR
	kf->Q[3][3] = 1;                         // PDR
										     
	kf->R[0][0] = 100;                       // GPSԤλ
	kf->R[1][1] = 100;					     // GPSԤλ
	kf->R[2][2] = 0.1;					     // GPS
	kf->R[3][3] = 1000;				         // GPS

	if (nmea->accuracy.err > 0 && 
		nmea->accuracy.err < 3.80 && 
		nmea->gga.hdop <= 0.4) 
	{
		kf->R[0][0] = 50;                    // GPS۲λ
		kf->R[1][1] = 50;					 // GPS۲λ
		kf->R[2][2] = 0.1;					 // GPS
		kf->R[3][3] = 50;					 // GPS
	}

	if (fabs(R2D((g_pdr->gpsHeading - g_pdr->lastGpsHeading))) > 50 ||
		g_pdr->gpsSpeed < 0.7 || g_pdr->gpsHeading == -1) {
		kf->R[3][3] = 10000;
	}
}

