/******************** (C) COPYRIGHT 2020 Geek************************************
* File Name          : Kalman.c
* Current Version    : V2.0
* Author             : logzhan
* Date of Issued     : 2022.10.15
* Comments           : PDRչ˲ں
********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "PDRBase.h"
#include "Kalman.h"
#include "Location.h"
#include "Matrix.h"
#include "Utils.h"

// չ
static double EkfVecBuf[5][N] = { {0.0} };
// չ󻺴
static double EkfMatBuf[7][N][N] = { { {0.0} } };

/**----------------------------------------------------------------------
* Function    : KalmanFilter_Init
* Description : ʼ˲
* Date        : 2022/09/16 logzhan
*---------------------------------------------------------------------**/
void EKF_Init(void)
{
	memset(EkfMatBuf, 0, sizeof(double) * 7 * N * N);
	memset(EkfVecBuf, 0, sizeof(double) * 5 * N);
}

/**----------------------------------------------------------------------
* Function    : EKFStatePredict
* Description : pdr˲״̬Ԥⷽ
* Date        : 2022/10/15 logzhan
*---------------------------------------------------------------------**/
void EKFStatePredict(EKFPara_t *ekf, double stepLen, PDR_t* pdr, int step) {

	double angle = pdr->deltaHeading + ekf->pXk[3];

	if(step == 0){
		ekf->pXk[3] = angle;
		return;
	}

	double(*Phi)[N] = (double(*)[N]) & (EkfMatBuf[0][0][0]);
	double(*pvec1)[N] = (double(*)[N]) & (EkfMatBuf[1][0][0]);
	double(*pvec2)[N] = (double(*)[N]) & (EkfMatBuf[2][0][0]);

	memset(EkfMatBuf, 0, sizeof(double) * 7 * N * N);

    // xk+1 = xk + step * cos(angle)   PDRλø·
	// yk+1 = yk + step * sin(angle)   PDRλø·
	// sk+1 = sk                       һͬ
	// ak+1 = ak * (angle / ak)  ak+1 = ak

	Phi[0][0] = 1;            Phi[1][0] = 0;          Phi[2][0] = 0;  Phi[3][0] = 0;
	Phi[0][1] = 0;			  Phi[1][1] = 1;		  Phi[2][1] = 0;  Phi[3][1] = 0;
	Phi[0][2] = cos(angle);	  Phi[1][2] = sin(angle); Phi[2][2] = 1;  Phi[3][2] = 0;
	Phi[0][3] = 0;            Phi[1][3] = 0;          Phi[2][3] = 0;  Phi[3][3] = 1;


	for (uchar i = 0; i < step; i++) {
		// logzhan 21/02/06:
		// phi[3][3]ΪkfԤÿζ¸ֵ, phi[3][3]ÿζ1ڹ
		// ¹УƫǵԤǹ̶ģʵʱûôãifжϿһ̶
		// Ԥֵ£ЧʱûиоԵĺ
		if (fabs(ekf->pXk[3]) > 0.000001 &&
			fabs(angle / ekf->pXk[3]) < 3)
		{
			Phi[3][3] = angle / ekf->pXk[3];
		}

		// ״̬·
		// pXk = phi * pXk
		VecMatMultiply(ekf->pXk, Phi, ekf->pXk);

		// Э : Pk = FPFt + Q
		MatrixMultiply(Phi, ekf->pPk, pvec1);               // F*P
		MatrixTrans(Phi, pvec2);                            // Ft
		MatrixMultiply(pvec1, pvec2, pvec1);                // F*P*Ft
		MatrixAdd(pvec1, ekf->Q, ekf->pPk);                 // F*P*Ft + Q
	}
}

/**----------------------------------------------------------------------
* Function    : EKFStateUpdate
* Description : չ˲״̬·
* Date        : 2020/7/22 logzhan
*---------------------------------------------------------------------**/
void EKFStateUpdate(EKFPara_t *kf, GNSS_t *pgnss,  Classifer_t *sys, PDR_t *pdr, 
	int scene_type) {

	double lambda = 0;

	// NxN
	double(*H)[N]     = (double(*)[N])&(EkfMatBuf[0][0][0]);
	double(*I)[N]     = (double(*)[N])&(EkfMatBuf[1][0][0]);
	double(*pvec1)[N] = (double(*)[N])&(EkfMatBuf[2][0][0]);
	double(*Ht)[N]    = (double(*)[N])&(EkfMatBuf[3][0][0]);
	double(*pvec3)[N] = (double(*)[N])&(EkfMatBuf[4][0][0]);
	double(*pvec4)[N] = (double(*)[N])&(EkfMatBuf[5][0][0]);
	double(*pv3)[N]   = (double(*)[N])&(EkfMatBuf[6][0][0]);

	memset(EkfMatBuf, 0, sizeof(double) * 7 * N * N);
	memset(EkfVecBuf, 0, sizeof(double) * 5 * N);

	// Nx1
	double(*pvec5)    = &(EkfVecBuf[1][0]);
	double(*pvec6)    = &(EkfVecBuf[2][0]);
	double(*pv1)      = &(EkfVecBuf[3][0]);
	double(*pv2)      = &(EkfVecBuf[4][0]);
	
	// λ
	for (char i = 0; i < 4; i++) {
		H[i][i] = 1;
		I[i][i] = 1;
	}

	double* z = &(EkfVecBuf[0][0]);
	// ȡ۲
	z[0] = pgnss->xNed;                                          // λ
	z[1] = pgnss->yNed;                                          // λ   
	z[2] = 0.5;                                                  // Ĭ0.5

	// ifʵʹGPSλñ仯Ͳ仯Ʋ, ⲿִ뻹û
	// ãʵڲ
	//if (pgnss->error > 0 && pgnss->error < 3.8 && pgnss->HDOP <= 0.4 && 
	//	g_pdr->deltaStepsPerGps != 0 && g_pdr->deltaStepsPerGps < 5) {
	//	// GPSٶȺͲƵƲ
	//	//if (g_pdr->motionFreq > 2.5 && (sys->type == 1)) {
	//	//	
	//	//	//printf("z[2] = %f\n", z[2]);
	//	//}
	//	z[2] = pgnss->vel * 0.514 / (g_pdr->deltaStepsPerGps);
	//}else {
	//	z[2] = kf->pXk[2];
	//}

	z[3] = pgnss->yaw * PI / 180;                                // ۲ĺΪGPSĺ

	//  K = K = PHt / (R + H*P*H^T)
	MatrixTrans(H, Ht);                                             // Ht
	MatrixMultiply(H, kf->pPk, pvec1);                             // HP
	MatrixMultiply(pvec1, Ht, pvec3);                               // pvec3 = H*P*H^T
	MatrixAdd(pvec3, kf->R, pvec4);                                 // pvec4 = R + H*P*H^T
	MatrixInverse(pvec4, pvec1);                                    //  1 / (R + H*P*H^T)

	for (char i = 0; i < N; i++) {
		for (uchar j = 0; j < N; j++) {
			pv3[i][j] = pvec3[i][j];
		}
	}
	MatrixMultiply(kf->pPk, Ht, pvec3);                                  // PHt
	MatrixMultiply(pvec3, pvec1, kf->Kk);                                 // K = PHt / (R + H*P*H^T)
								                                  
								                                  
	VecMatMultiply(kf->pXk, H, pvec5);                                   // pvec5 = Hx'
	VectorSub(z, pvec5, pvec6);                                       // pvec6 = Z - Hx'
	modAngle(&pvec6[3], -PI, PI);
	for (char i = 0; i < N; i++) {
		pv1[i] = pvec6[i];
	}
	
	VecMatMultiply(pvec6, kf->Kk, pvec5); // pvec5 = K*( z - Hx')
	// Get the optimally updated state estimation
	VectorAdd(kf->pXk, pvec5, kf->Xk);

	// ---- prepare the matrix for updated estimate covariance ------
	// pvec1 = K*H
	MatrixMultiply(kf->Kk, H, pvec1);
	// pvec2 = (I - K*H)
	MatrixSub(I, pvec1, Ht);
	// pvec3 = (I - K*H)*P
	MatrixMultiply(Ht, kf->pPk, pvec3);
	// pvec4 = (I- K*H)^T
	MatrixTrans(Ht, pvec4);
	// pvec1 = (I - K*H)*P*(I- K*H)^T
	MatrixMultiply(pvec3, pvec4, pvec1);

	// pvec2 = K*R
	MatrixMultiply(kf->Kk, kf->R, Ht);
	// pvec3 = K^T
	MatrixTrans(kf->Kk, pvec3);
	// pvec4 = K*R*K^T
	MatrixMultiply(Ht, pvec3, pvec4);
	// Get the updated estimate covariance: P' = (I - K*H)*P*(I- K*H)^T + K*R*K^T
	MatrixAdd(pvec1, pvec4, kf->Pk);

	// pv2 = (z - Hx')*( H*P*H^T )
	VecMatMultiply(pv1, pv3, pv2);

	// inner product the pre-fit residual and the transitioned residual: lamda = (z - Hx')*( H*P*H^T )  .*  (z-Hx')
	// Calculate the difference between original differenc and the transitioned,
	// if the value is close to zero, it shows that there is a big gap in the 
	// oprimization. Thus, we believe our PDR algorithm
	for (char i = 0; i < N; i++) {
		lambda += pv2[i] * pv1[i];
	}

	kf->Lambda = lambda;
	kf->pLat = pgnss->lat;
	kf->pLon = pgnss->lon;

	if (lambda >= 200)return;

	for (char i = 0; i < N; i++) {
		kf->Xk[i] = kf->pXk[i];
		for (int j = 0; j < N; j++) {
			kf->Pk[i][j] = kf->pPk[i][j];
		}
	}
	
}

/**----------------------------------------------------------------------
* Function    : EKFCalQRMatrix
* Description : GPSź˲qǹ
*               ҪߵԤλþءrΪGNSS۲GPS
*               Ϣйء
* Date        : 2022/09/19 logzhan
*---------------------------------------------------------------------**/
void EKFCalQRMatrix(Nmea_t *nmea, PDR_t *pdr, Classifer_t *sys,
	EKFPara_t *kf) {

	if (pdr->MotionType != PDR_MOTION_TYPE_HANDHELD && 
		pdr->MotionType != PDR_MOTION_TYPE_STATIC) {
		kf->Q[0][0] = 25;                   // PDRԤλ
		kf->Q[1][1] = 25;                   // PDRԤλ
		kf->Q[2][2] = 1;                    // PDR
		kf->Q[3][3] = 100;                  // PDR

		kf->R[0][0] = 50;                    // GPS۲λ
		kf->R[1][1] = 50;					 // GPS۲λ
		kf->R[2][2] = 0.1;					 // GPS
		kf->R[3][3] = 50;					 // GPS
		return;
	}

	kf->Q[0][0] = 0.1;                       // PDRԤλ
	kf->Q[1][1] = 0.1;                       // PDRԤλ
	kf->Q[2][2] = 1;                         // PDR
	kf->Q[3][3] = 1;                         // PDR
										     
	kf->R[0][0] = 100;                       // GPSԤλ
	kf->R[1][1] = 100;					     // GPSԤλ
	kf->R[2][2] = 0.1;					     // GPS
	kf->R[3][3] = 1000;				         // GPS

	if (nmea->accuracy.err > 0 && 
		nmea->accuracy.err < 3.80 && 
		nmea->gga.hdop <= 0.4) 
	{
		kf->R[0][0] = 50;                    // GPS۲λ
		kf->R[1][1] = 50;					 // GPS۲λ
		kf->R[2][2] = 0.1;					 // GPS
		kf->R[3][3] = 50;					 // GPS
	}

	if (fabs(R2D((pdr->GpsHeading - pdr->LastGpsHeading))) > 50 ||
		pdr->GpsSpeed < 0.7 || pdr->GpsHeading == -1) {
		kf->R[3][3] = 10000;
	}
}

