﻿/******************** (C) COPYRIGHT 2023 Geek************************************
* File Name          : Pedometer.c
* Department         : Geek Technology
* Current Version    : V1.0.0
* Author             : zhanli
* Date of Issued     : 2023.02.24
* Comments           : PDR计步器相关
********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "Pedometer.h"

const static char* Pedometer_Version = "Pedometer.1.0.0.20230224";

static void AccBufferUpdate(double ax, double ay, double az);
static void StateDetector(unsigned long long timeStamp);
static void Pedometer_Filter(void);
static void Steps_Feature(void);
static void Steps_Recording(unsigned long long timeStamp);

static float     DetVehicleAccBuff[DET_VEHICLE_ACC_BUFF_LEN] = { 0 };
static float     lastAccZ = 0;
static uint32_t  ReceivedHowManyAcc = 0;
static double    SumAcc = 0;

static double DifFiltCoefficient1[5]  = { 0.0123456790123457, 0.0493827160493827, 0.123456790123457, 0.197530864197531, 
                                          0.234567901234568 };

static double DifFiltCoefficient2[9]  = { 0.0016, 0.0064, 0.016, 0.032, 0.056, 0.0832, 0.1088, 0.128, 0.136 };
static double DifFiltCoefficient3[13] = { 0.00041649312786339, 0.00166597251145356, 0.0041649312786339, 0.0083298625572678, 
                                          0.0145772594752187,  0.0233236151603498,  0.0349854227405248, 0.0483132028321533, 
	                                      0.0620574760516451,  0.0749687630154102,  0.0857975843398584, 0.0932944606413994, 
	                                      0.0962099125364431 };

static double DifFiltCoefficient4[17] = { 0.000152415790275873, 0.00060966316110349, 0.00152415790275873, 0.00304831580551745,
                                          0.00533455265965554,  0.00853528425544886, 0.0128029263831733,  0.0182898948331047, 
	                                      0.0251486053955190,   0.0329218106995885,  0.0411522633744856,  0.0493827160493827, 
	                                      0.0571559213534522,   0.0640146319158665,  0.0695016003657979,  0.0731595793324188, 
	                                      0.0745313214449017 };

static double DifFiltOutput[FILTER_AXIS_NUM][5];
static double OutputFilt[FILTER_AXIS_NUM][2];
static double StepLcVar[FILTER_AXIS_NUM][3];                        // 仅用于计步器特征提取

static double        AccBuffer[AXIS_NUM][ACC_BUFF_NUM];             // 经过降采样和5点均值滤波后的Acc缓存,长度为39

// 计步器特征
static char          StepReFeature[FILTER_AXIS_NUM] = { 0 };
static double        FeatureAmp[FILTER_AXIS_NUM][8];                // Acc的振幅特征
static uint16_t      FeatureTime[FILTER_AXIS_NUM][8];
				     
static double        StepRecordeState[AXIS_ALL][4];
static double        StepRecordTime[AXIS_ALL][9];                   // 记录计步器的状态时间，记录波峰和波谷
static uint64_t      TimeStampSave[AXIS_ALL][2];
				     
static uint16_t      StepMaxTime = 100;
static uint16_t      StepMinTime = 10;
				     
static uint64_t      timeStamp = 0;
static uint64_t      timeStamp_in_vehicle = 0;
static char          Flag_whether_in_vehicle = 0;
static char          Flag_step_exist = 0;
static double        AvergeFiltSave[AXIS_NUM - 1][AVG_FILTER_BUFF_LEN];
static double        StateDetectLcVar[5] = { 0 };
static StepsDefine_t Steps      = { 0, 0 };
static stepsRe       ReSteps    = { 0 };
static DownSample_t  DownSample = { 0, 0, 0, 0 };
static StepState_t   StateLcVar[2]  = { NonSteps, NonSteps };
stateNowDefine       stateNow;
StateDef_t           Restate;

/**---------------------------------------------------------------------
* Function    : Pedometer_GetVersion
* Description : 获取计步器版本
* Date        : 2023/02/24 
*---------------------------------------------------------------------**/
const char* Pedometer_GetVersion(void){
	return Pedometer_Version;
}

/**---------------------------------------------------------------------
* Function    : initPedometer
* Description : PDR 初始化计步器
* Date        : 2020/2/20 logzhan
*---------------------------------------------------------------------**/
void Pedometer_Init(void)        
{
	STEPLIB_MEMSET(AccBuffer, 0, sizeof(AccBuffer));
	STEPLIB_MEMSET(DifFiltOutput, 0, sizeof(DifFiltOutput));
	STEPLIB_MEMSET(OutputFilt, 0, sizeof(OutputFilt));
	STEPLIB_MEMSET(StepLcVar, 0, sizeof(StepLcVar));
	STEPLIB_MEMSET(StepReFeature, 0, sizeof(StepReFeature));
	STEPLIB_MEMSET(FeatureAmp, 0, sizeof(FeatureAmp));
	STEPLIB_MEMSET(FeatureTime, 0, sizeof(FeatureTime));
	STEPLIB_MEMSET(StepRecordeState, 0, sizeof(StepRecordeState));
	STEPLIB_MEMSET(StepRecordTime, 0, sizeof(StepRecordTime));
	STEPLIB_MEMSET(TimeStampSave, 0, sizeof(TimeStampSave));
	STEPLIB_MEMSET(&DownSample, 0, sizeof(DownSample));
	SumAcc = 0;
	timeStamp = 0;
	STEPLIB_MEMSET(&Steps, 0, sizeof(Steps));
	STEPLIB_MEMSET(&ReSteps, 0, sizeof(ReSteps));
	STEPLIB_MEMSET(StateDetectLcVar, 0, sizeof(StateDetectLcVar));
	STEPLIB_MEMSET(StateLcVar, 0, sizeof(StateLcVar));
	STEPLIB_MEMSET(&stateNow, 0, sizeof(stateNow));
	STEPLIB_MEMSET(&Restate, 0, sizeof(Restate));
	STEPLIB_MEMSET(AvergeFiltSave, 0, sizeof(AvergeFiltSave));
	ReceivedHowManyAcc = 0;
	STEPLIB_MEMSET(DetVehicleAccBuff, 0, sizeof(DetVehicleAccBuff));
	timeStamp_in_vehicle = 0;
	Flag_whether_in_vehicle = 0;
	Flag_step_exist = 0;

}

/**---------------------------------------------------------------------
* Function    : reinitPedometer
* Description : PDR 重新初始化计步器
* Date        : 2020/2/20 logzhan
*---------------------------------------------------------------------**/
void Pedometer_ReInit(void)
{
	timeStamp = 0;
	STEPLIB_MEMSET(&Steps, 0, sizeof(Steps));
	STEPLIB_MEMSET(&ReSteps, 0, sizeof(ReSteps));
}

/**---------------------------------------------------------------------
* Function    : Pedometer_Update
* Description : PDR 计步器的加速度输入, 并通过指针返回用户步数
* Date        : 2023/02/25 logzhan
*---------------------------------------------------------------------**/
void Pedometer_Update(IMU_t* imu, uint32_t* step)
{
	unsigned char k = 0;

	// 检测当前是否属于开车状态, 这个函数也是相对耗时的, 这个函数只用
	// 了加速度计的z轴, 是不是存在问题？
	DetVehicleMode(imu->acc.s[0], imu->acc.s[1], imu->acc.s[2]);

	timeStamp += ACC_SAMPLE_TIME;
	DownSample.ax += imu->acc.s[0] / 9.8;
	DownSample.ay += imu->acc.s[1] / 9.8;
	DownSample.az += imu->acc.s[2] / 9.8;
	DownSample.flag++;

	if (DownSample.flag == DOWN_SAMPLE_NUM)
	{
		if (Restate.startTimeStamp == 0)
		{
			Restate.startTimeStamp = timeStamp;
		}
		// 降采样的数据用四个点的均值滤波
		DownSample.ax /= DOWN_SAMPLE_NUM;
		DownSample.ay /= DOWN_SAMPLE_NUM;
		DownSample.az /= DOWN_SAMPLE_NUM;

		// 均值滤波器缓存更新，得到25Hz的Acc缓存
		for (k = 0; k < AVG_FILTER_BUFF_LEN - 1; k++)
		{
			AvergeFiltSave[0][k] = AvergeFiltSave[0][k + 1];
			AvergeFiltSave[1][k] = AvergeFiltSave[1][k + 1];
			AvergeFiltSave[2][k] = AvergeFiltSave[2][k + 1];
		}
		AvergeFiltSave[0][k] = DownSample.ax;
		AvergeFiltSave[1][k] = DownSample.ay;
		AvergeFiltSave[2][k] = DownSample.az;

		// 对均值滤波器中的5个点再求均值处理，降低噪声
		STEPLIB_MEMSET(&DownSample, 0, sizeof(DownSample));
		for (k = 0; k < AVG_FILTER_BUFF_LEN; k++)
		{
			DownSample.ax += AvergeFiltSave[0][k];
			DownSample.ay += AvergeFiltSave[1][k];
			DownSample.az += AvergeFiltSave[2][k];
		}
		DownSample.ax /= AVG_FILTER_BUFF_LEN;
		DownSample.ay /= AVG_FILTER_BUFF_LEN;
		DownSample.az /= AVG_FILTER_BUFF_LEN;

		// 计步器缓存Buffer更新
		AccBufferUpdate(DownSample.ax, DownSample.ay, DownSample.az);
		// 计步器状态检测
		StateDetector(timeStamp);
		
		if (StateDetectLcVar[4] < 0.5l)
		{
			// 计步器多通道滤波处理
			Pedometer_Filter();
			// 计步器波峰波谷以及振幅信息提取
			Steps_Feature();
			// 步数记录
			Steps_Recording(timeStamp);
		}
		// 重置DownSample标志位
		STEPLIB_MEMSET(&DownSample, 0, sizeof(DownSample));
	}
	// 如果计步标志位不存在，那么不计步数
	if (Flag_step_exist == 0){
		// vehicle  for debug
		timeStamp = timeStamp + 1; 
	}
	else if (llabs((long long)(timeStamp - timeStamp_in_vehicle)) > 500)
	{
		// 更新步数
		*step = ReSteps.walkSteps;
		Flag_step_exist = 1;
	}
}

/**---------------------------------------------------------------------
* Function    : detVehicleMode
* Description : PDR 检测汽车模式, 如果是检测到汽车模式，那么不计步数
* Date        : 2020/2/1 logzhan
*---------------------------------------------------------------------**/
void DetVehicleMode(double ax, double ay, double az) 
{
	int t;
	int index_move;

	int num_SumAcc_Ascend_Descend[DET_VEHICLE_ACC_BUFF_LEN / 2] = { 0 };
	int index_SumAcc_Ascend_Descend = 0;
	int num_SumAcc_Ascend_Descend_occur = 0;
	char flag_1_ascend_0_descend = 0;


	if (ReceivedHowManyAcc < DET_VEHICLE_ACC_BUFF_LEN)
	{
		ReceivedHowManyAcc++;
		DetVehicleAccBuff[ReceivedHowManyAcc - 1] = (float)az - lastAccZ;
		lastAccZ = (float)az;
	}
	else
	{
		for (index_move = 0; index_move < DET_VEHICLE_ACC_BUFF_LEN - 1; index_move++)
		{
			DetVehicleAccBuff[index_move] = DetVehicleAccBuff[index_move + 1];
		}

		DetVehicleAccBuff[DET_VEHICLE_ACC_BUFF_LEN - 1] = (float)az - lastAccZ;
		lastAccZ = (float)az;
	}

	if (ReceivedHowManyAcc >= DET_VEHICLE_ACC_BUFF_LEN)
	{
		// The Number of Sign Change of First Order Difference,to find the window_length of window_min2max

		for (t = 1; t < WINDOW_LENGTH_MIN2MAX; t++)
		{
			if (DetVehicleAccBuff[t] > 0 && DetVehicleAccBuff[t - 1] < 0)
			{
				flag_1_ascend_0_descend = 1;
				index_SumAcc_Ascend_Descend++;
			}
			else if (DetVehicleAccBuff[t - 1] > 0 && DetVehicleAccBuff[t] < 0)
			{
				flag_1_ascend_0_descend = 0;
				index_SumAcc_Ascend_Descend++;
			}

			if (flag_1_ascend_0_descend == 1)
			{
				num_SumAcc_Ascend_Descend[index_SumAcc_Ascend_Descend]++;
			}
			else if (flag_1_ascend_0_descend == 0)
			{
				num_SumAcc_Ascend_Descend[index_SumAcc_Ascend_Descend]--;
			}
		}


		for (t = 0; t < DET_VEHICLE_ACC_BUFF_LEN / 2 - 2; t++)
		{
			index_move = abs(num_SumAcc_Ascend_Descend[t]);

			if (index_move >= 6)
			{
				num_SumAcc_Ascend_Descend_occur++;
				if (num_SumAcc_Ascend_Descend_occur >= 3)
				{
					break;
				}
			}
		}

		if (num_SumAcc_Ascend_Descend_occur < 3)
		{
			timeStamp_in_vehicle = timeStamp;
			Flag_step_exist = 0;  //in  vehicle
		}
		else
		{
			Flag_step_exist = 1;  //count step
		}
	}
	else
	{
		Flag_step_exist = 1;   // start 2 seconds
	}

}

/**---------------------------------------------------------------------
* Function    : AccBufferUpdate
* Description : 更新Acc的Buffer缓存
* Date        : 2023/02/25
*---------------------------------------------------------------------**/
static void AccBufferUpdate(double ax, double ay, double az)
{
	for (uint8_t i = 0; i < AXIS_NUM; i++){
		for (uint8_t j = 0; j < (ACC_BUFF_NUM - 1); j++){
			AccBuffer[i][j] = AccBuffer[i][j + 1];
		}
	}
	AccBuffer[0][ACC_BUFF_NUM - 1] = ax;
	AccBuffer[1][ACC_BUFF_NUM - 1] = ay;
	AccBuffer[2][ACC_BUFF_NUM - 1] = az;
	AccBuffer[3][ACC_BUFF_NUM - 1] = sqrt(ax*ax + ay*ay + az*az);
}

/**---------------------------------------------------------------------
* Function    : Pedometer_Filter
* Description : 计步器滤波器,一共有4个频段的低通滤波器。针对x,y,z,mod(x,y,z)
*               的数据进行滤波处理，一共得到16轴的输出数据。
* Date        : 2023/02/25
*---------------------------------------------------------------------**/
static void Pedometer_Filter(void)
{
	short i, j;
	for (i = 0; i < FILTER_AXIS_NUM; i++){
		DifFiltOutput[i][0] = DifFiltOutput[i][1];
		DifFiltOutput[i][1] = DifFiltOutput[i][2];
		DifFiltOutput[i][2] = DifFiltOutput[i][3];
		DifFiltOutput[i][3] = DifFiltOutput[i][4];
		OutputFilt[i][0]    = OutputFilt[i][1];
	}

	for (i = 0; i < AXIS_NUM; i++){
		double FiltSum = 0;
		for (j = 0; j < 4; j++)
		{
			FiltSum += DifFiltCoefficient1[j] * (AccBuffer[i][15 + j] + AccBuffer[i][23 - j]);
		}
		FiltSum += DifFiltCoefficient1[4] * AccBuffer[i][19];
		DifFiltOutput[i][4] = (FiltSum);

		FiltSum = 0;
		for (j = 0; j < 8; j++)
		{
			FiltSum += DifFiltCoefficient2[j] * (AccBuffer[i][11 + j] + AccBuffer[i][27 - j]);
		}
		FiltSum += DifFiltCoefficient2[8] * AccBuffer[i][19];
		DifFiltOutput[i + AXIS_NUM][4] = (FiltSum);

		FiltSum = 0;
		for (j = 0; j < 12; j++){
			FiltSum += DifFiltCoefficient3[j] * (AccBuffer[i][7 + j] + AccBuffer[i][31 - j]);
		}
		FiltSum += DifFiltCoefficient3[12] * AccBuffer[i][19];
		DifFiltOutput[i + AXIS_NUM * 2][4] = (FiltSum);

		FiltSum = 0;
		for (j = 0; j < 16; j++){
			FiltSum += DifFiltCoefficient4[j] * (AccBuffer[i][3 + j] + AccBuffer[i][35 - j]);
		}
		FiltSum += DifFiltCoefficient4[16] * AccBuffer[i][19];
		DifFiltOutput[i + AXIS_NUM * 3][4] = (FiltSum);
	}
	// 更新16个轴计步器的输出结果
	for (i = 0; i < FILTER_AXIS_NUM; i++){
		OutputFilt[i][1] = DifFiltOutput[i][4];
	}
}
/**---------------------------------------------------------------------
* Function    : Steps_Feature
* Description : 提取计步器特征，分析计步器的波峰波谷以及振幅情况
* Date        : 2023/02/25
*---------------------------------------------------------------------**/
static void Steps_Feature(void)
{
	// 遍历16个滤波器通道输出
	for (short axis = 0; axis < FILTER_AXIS_NUM; axis++)
	{
		StepLcVar[axis][0] = StepLcVar[axis][0] + 1.0;
		
		int8_t FeatureType = 0;
		// 推测是判断遇到波峰或者波谷
		if ((StepLcVar[axis][1] < 1) && ((OutputFilt[axis][1] > OutputFilt[axis][0]) || 
			((OutputFilt[axis][1] == OutputFilt[axis][0]) && (StepLcVar[axis][1] == 1)))){
			FeatureType = 1;
		}

		// 推测是判断遇到波峰或者波谷
		if ((StepLcVar[axis][1] > -1) && ((OutputFilt[axis][1] < OutputFilt[axis][0]) || 
			((OutputFilt[axis][1] == OutputFilt[axis][0]) && (StepLcVar[axis][1] == -1)))){
			FeatureType = -1;
		}

		// 如果遇到的了波峰和波谷
		if (FeatureType != 0) {
			// 更新振幅特征和时间特征(记录8个点)数组
			for (uint8_t ch = 0; ch < 7; ch++) {
				FeatureAmp[axis][ch] = FeatureAmp[axis][ch + 1];
				FeatureTime[axis][ch] = FeatureTime[axis][ch + 1];
			}
			// 更新时间特征和振幅特征的最新值，这里根据波峰和波谷做取反处理
			FeatureAmp[axis][7] = (StepLcVar[axis][2] - OutputFilt[axis][0]) * FeatureType;
			FeatureTime[axis][7] = (uint16_t)(StepLcVar[axis][0]);

			StepLcVar[axis][2] = OutputFilt[axis][0];
			// 保存结果
			StepLcVar[axis][0] = 0;
			StepLcVar[axis][1] = FeatureType;
			StepReFeature[axis] = STEP_TRUE;
		}
	}
}

static void Steps_Recording(unsigned long long timeStamp)
{
	short  nB, nC;
	double thred, nD = 0, nE = 0, Differ_Sum = 0;
	double StepAmp12, StepAmp34, StepAmp56, StepAmp78, StepAmp14, StepAmp58, StepAmp18;
	unsigned short StepTime12, StepTime34, StepTime56, StepTime78, StepTime14, StepTime58, StepTime38, StepTime18;

	// 从第4个轴到第20个轴进行遍历
	for (short axis = AXIS_NUM; axis < AXIS_ALL; axis++){
		// StepRecordTime的准确含义是什么？
		if (StepRecordTime[axis][4] > 5 && axis != FILTER_AXIS_NUM){
			nD = StepRecordTime[axis][5] / StepRecordTime[axis][4];
		}
	}
	// 如果nD大于0
	if (nD > 0)
	{
		// 从0到2轴(x,y,z)进行相同的遍历
		for (nC = 0; nC < AXIS_NUM - 1; nC++)
		{
			if ((StepRecordTime[nC][4] > 5) && (nD > StepRecordTime[nC][5] / StepRecordTime[nC][4]))
			{
				nD = StepRecordTime[nC][5] / StepRecordTime[nC][4];
			}
		}
		// 最后才计算FILTER_AXIS_NUM的结果
		nC = FILTER_AXIS_NUM;
		if ((StepRecordTime[nC][4] > 5) && (nD > StepRecordTime[nC][5] / StepRecordTime[nC][4]))
		{
			nD = StepRecordTime[nC][5] / StepRecordTime[nC][4];
		}
	}

	// 求加速度计的差分和
	Differ_Sum = 0;
	for (uint8_t j = 0; j < ACC_BUFF_NUM; j++){
		Differ_Sum = Differ_Sum + fabs(AccBuffer[3][j + 1] - AccBuffer[3][j]);
	}

	nC = 3;  
	if ((StepRecordTime[0][4] > 5) && (StepRecordTime[1][4] > 5) && (StepRecordTime[2][4] > 5)
		&& (StepRecordTime[0][5] / StepRecordTime[0][4] + StepRecordTime[1][5] / StepRecordTime[1][4] + StepRecordTime[2][5] / StepRecordTime[2][4]
		> 5.7 * StepRecordTime[nC][5] / StepRecordTime[nC][4]) && (Differ_Sum < 1.2))   //  >5
	{

	}else{
		if ((StepRecordTime[nC][4] > 5) && (nD > StepRecordTime[nC][5] / StepRecordTime[nC][4])){
			nD = StepRecordTime[nC][5] / StepRecordTime[nC][4];
		}
	}
	// 对滤波器输出的16个轴进行处理
	for (uint8_t axis = 0; axis < FILTER_AXIS_NUM; axis++){
		if (StepReFeature[axis] == STEP_FALSE) {
			continue;
		}

		// 计算计步器的振幅特征和时间特征
		StepAmp12 = FeatureAmp[axis][0] + FeatureAmp[axis][1];
		StepAmp34 = FeatureAmp[axis][2] + FeatureAmp[axis][3];
		StepAmp56 = FeatureAmp[axis][4] + FeatureAmp[axis][5];
		StepAmp78 = FeatureAmp[axis][6] + FeatureAmp[axis][7];
		StepAmp14 = StepAmp12 + StepAmp34;
		StepAmp58 = StepAmp56 + StepAmp78;
		StepAmp18 = StepAmp14 + StepAmp58;

		StepTime12 = FeatureTime[axis][0] + FeatureTime[axis][1];
		StepTime34 = FeatureTime[axis][2] + FeatureTime[axis][3];
		StepTime56 = FeatureTime[axis][4] + FeatureTime[axis][5];
		StepTime78 = FeatureTime[axis][6] + FeatureTime[axis][7];
		StepTime14 = StepTime12 + StepTime34;
		StepTime58 = StepTime56 + StepTime78;
		StepTime38 = StepTime34 + StepTime58;
		StepTime18 = StepTime14 + StepTime58;

		if (StepRecordeState[axis][2] < 30)
		{
			StepRecordeState[axis][2] = StepRecordeState[axis][2] + 0.51;
			StepRecordTime[axis][2]   = StepRecordTime[axis][2] + FeatureTime[axis][7];
			StepRecordeState[axis][3] = StepRecordeState[axis][3] + 0.51;
			StepRecordTime[axis][3]   = StepRecordTime[axis][3] + FeatureTime[axis][7];

			if ((axis % 4 == 3) && (StepRecordeState[FILTER_AXIS_NUM + axis / 4][2] < 30))
			{
				StepRecordeState[FILTER_AXIS_NUM + axis / 4][2] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][2] + 0.51;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][2]   = StepRecordTime[FILTER_AXIS_NUM + axis / 4][2] + FeatureTime[axis][7];
				StepRecordeState[FILTER_AXIS_NUM + axis / 4][3] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][3] + 0.51;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][3]   = StepRecordTime[FILTER_AXIS_NUM + axis / 4][3] + FeatureTime[axis][7];
			}
		}

		// 动态阈值
		thred = 0.015 + ((FILTER_AXIS_NUM - axis - 1) / AXIS_NUM) * 0.037;

		if (StepRecordTime[axis][6] < 60) {
			StepRecordTime[axis][6]++;
		}else{
			StepRecordTime[axis][4] = 0;
			StepRecordTime[axis][5] = 0;
			StepRecordTime[axis][7] = 0;
		}

		// 如果轴是XYZ和合成轴
		if (axis % 4 == AXIS_XYZ)
		{
			if (StepRecordTime[FILTER_AXIS_NUM + axis / 4][6] < 25){
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][6]++;
			}else{
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][4] = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][5] = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][7] = 0;
			}

			if ((fabs(FeatureAmp[axis][2] - FeatureAmp[axis][3]) / (StepAmp34) < 0.15l)
				&& (fabs(FeatureAmp[axis][3] - FeatureAmp[axis][4]) / (FeatureAmp[axis][3] + FeatureAmp[axis][4]) < 0.15l)
				&& (fabs(FeatureAmp[axis][4] - FeatureAmp[axis][5]) / (StepAmp56) < 0.15l)
				&& (fabs(FeatureAmp[axis][5] - FeatureAmp[axis][6]) / (FeatureAmp[axis][5] + FeatureAmp[axis][6]) < 0.15l)
				&& (fabs(FeatureAmp[axis][6] - FeatureAmp[axis][7]) / (StepAmp78) < 0.15l)
				&& (FeatureAmp[axis][2] > thred) && (FeatureAmp[axis][3] > thred) && (FeatureAmp[axis][4] > thred) && (FeatureAmp[axis][5] > thred) && (FeatureAmp[axis][6] > thred) && (FeatureAmp[axis][7] > thred)
				&& (ABS_INT(StepTime34 - StepTime56) / (StepTime34 + StepTime56) < 0.15l)
				&& (ABS_INT(StepTime56 - StepTime78) / (StepTime56 + StepTime78) < 0.15)
				&& (((2 * FeatureTime[axis][2] > FeatureTime[axis][3]) && (FeatureTime[axis][2] < 2 * FeatureTime[axis][3])) || (ABS_INT(FeatureTime[axis][2] - FeatureTime[axis][3]) <= 3))
				&& (((2 * FeatureTime[axis][4] > FeatureTime[axis][5]) && (FeatureTime[axis][4] < 2 * FeatureTime[axis][5])) || (ABS_INT(FeatureTime[axis][4] - FeatureTime[axis][5]) <= 3))
				&& (((2 * FeatureTime[axis][6] > FeatureTime[axis][7]) && (FeatureTime[axis][6] < 2 * FeatureTime[axis][7])) || (ABS_INT(FeatureTime[axis][6] - FeatureTime[axis][7]) <= 3))
				&& (StepTime58 > StepMinTime)
				&& (StepTime58 < StepMaxTime))
			{
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][8] = PEDOMETER_FREQ * 3.0 / StepTime38;
				TimeStampSave[FILTER_AXIS_NUM + axis / 4][1] = timeStamp;
				if (StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] == 0)
				{
					StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] + 3;
					StepRecordTime[FILTER_AXIS_NUM + axis / 4][0] = StepRecordTime[FILTER_AXIS_NUM + axis / 4][0] + StepTime38;
					StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] + 3;
					StepRecordTime[FILTER_AXIS_NUM + axis / 4][1] = StepRecordTime[FILTER_AXIS_NUM + axis / 4][1] + StepTime38;
					TimeStampSave[FILTER_AXIS_NUM + axis / 4][0] = timeStamp - (unsigned long)(StepTime38 * 1000 / PEDOMETER_FREQ);
				}
				else
				{
					if (((ABS_INT(FeatureTime[axis][2] - FeatureTime[axis][4]) < 0.2 * (FeatureTime[axis][2] + FeatureTime[axis][4])) || (ABS_INT(FeatureTime[axis][2] - FeatureTime[axis][4]) < 5))
						&& ((ABS_INT(FeatureTime[axis][3] - FeatureTime[axis][5]) < 0.2 * (FeatureTime[axis][3] + FeatureTime[axis][5])) || (ABS_INT(FeatureTime[axis][3] - FeatureTime[axis][5]) < 5))
						&& ((ABS_INT(FeatureTime[axis][4] - FeatureTime[axis][6]) < 0.2 * (FeatureTime[axis][4] + FeatureTime[axis][6])) || (ABS_INT(FeatureTime[axis][4] - FeatureTime[axis][6]) < 5))
						&& ((ABS_INT(FeatureTime[axis][5] - FeatureTime[axis][7]) < 0.2 * (FeatureTime[axis][5] + FeatureTime[axis][7])) || (ABS_INT(FeatureTime[axis][5] - FeatureTime[axis][7]) < 5)))
					{

						if ((StepRecordTime[FILTER_AXIS_NUM + axis / 4][7] > 0) && (fabs(StepTime38 / 3.0 - StepRecordTime[FILTER_AXIS_NUM + axis / 4][7]) > 0.2 * StepRecordTime[FILTER_AXIS_NUM + axis / 4][7]))
						{
							StepRecordTime[FILTER_AXIS_NUM + axis / 4][4] = 0;
							StepRecordTime[FILTER_AXIS_NUM + axis / 4][5] = 0;
						}
						if (StepRecordTime[FILTER_AXIS_NUM + axis / 4][6] >= 6)
						{
							StepRecordTime[FILTER_AXIS_NUM + axis / 4][4] = StepRecordTime[FILTER_AXIS_NUM + axis / 4][4] + 3;
							StepRecordTime[FILTER_AXIS_NUM + axis / 4][5] = StepRecordTime[FILTER_AXIS_NUM + axis / 4][5] + StepTime38;
							StepRecordTime[FILTER_AXIS_NUM + axis / 4][6] = 0;
						}
						StepRecordTime[FILTER_AXIS_NUM + axis / 4][7] = StepTime38 / 3.0;
					}
					if (StepRecordeState[FILTER_AXIS_NUM + axis / 4][3] <= 3)
					{
						StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] + StepRecordeState[FILTER_AXIS_NUM + axis / 4][3];
						StepRecordTime[FILTER_AXIS_NUM + axis / 4][0] = StepRecordTime[FILTER_AXIS_NUM + axis / 4][0] + StepRecordTime[FILTER_AXIS_NUM + axis / 4][3];
					}
					else
					{
						StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] + 3;
						StepRecordTime[FILTER_AXIS_NUM + axis / 4][0] = StepRecordTime[FILTER_AXIS_NUM + axis / 4][0] + StepTime38;
					}


					if (StepRecordeState[FILTER_AXIS_NUM + axis / 4][3] <= 3)
					{
						StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] + StepRecordeState[FILTER_AXIS_NUM + axis / 4][3];
					}
					else if ((StepRecordTime[FILTER_AXIS_NUM + axis / 4][3] - StepTime38 > StepMaxTime)
						&& (StepRecordeState[FILTER_AXIS_NUM + axis / 4][3] > 11))
					{
						StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] + 3;
					}
					else
					{
						if (StepRecordTime[FILTER_AXIS_NUM + axis / 4][4] > 0)
						{
							nE = StepRecordTime[FILTER_AXIS_NUM + axis / 4][3] * StepRecordTime[FILTER_AXIS_NUM + axis / 4][4] / StepRecordTime[FILTER_AXIS_NUM + axis / 4][5];
						}
						if ((StepRecordTime[FILTER_AXIS_NUM + axis / 4][4] > 0) && (nE < StepRecordeState[FILTER_AXIS_NUM + axis / 4][3]))
						{
							if (nE > 4)
							{
								StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] + nE;
							}
							else
							{
								StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] + 3;
							}
						}
						else
						{
							if (StepRecordeState[FILTER_AXIS_NUM + axis / 4][3] > 4)
							{
								StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] + StepRecordeState[FILTER_AXIS_NUM + axis / 4][3];
							}
							else
							{
								StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] = StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] + 3;
							}
						}
					}

					StepRecordTime[FILTER_AXIS_NUM + axis / 4][1] = StepRecordTime[FILTER_AXIS_NUM + axis / 4][1] + StepRecordTime[FILTER_AXIS_NUM + axis / 4][3];
				}
				StepRecordeState[FILTER_AXIS_NUM + axis / 4][2] = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][2] = 0;
				StepRecordeState[FILTER_AXIS_NUM + axis / 4][3] = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][3] = 0;
			}
			else if ((StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] > 0) && ((StepRecordTime[FILTER_AXIS_NUM + axis / 4][2] >= StepMaxTime * 2) || (StepAmp18 < 4 * thred) || (StateDetectLcVar[2] > PEDOMETER_FREQ)))
			{
				nB = AXIS_ALL - 1;
				if (nD > 0)
				{
					for (nC = (AXIS_ALL - 1); nC >= 0; nC--)
					{
						if (StepRecordTime[nB][0] < StepRecordTime[nC][0])
						{
							nB = nC;
						}
					}
				}
				else
				{
					for (nC = (AXIS_ALL - 1); nC >= AXIS_NUM; nC--)
					{
						if ((StepRecordTime[nB][0] < StepRecordTime[nC][0]) && (nC != FILTER_AXIS_NUM))
						{
							nB = nC;
						}
					}
				}
				if ((nB == FILTER_AXIS_NUM + axis / 4) && (StepRecordeState[nB][0] >= 8))
				{
					if (Steps.walkRealTime > StepRecordeState[nB][1])
					{
						Steps.walk = Steps.walk + Steps.walkRealTime;
					}
					else
					{
						Steps.walk = Steps.walk + StepRecordeState[nB][1];
					}
					Steps.walkRealTime = 0;

					ReSteps.walkSteps = (unsigned long)(Steps.walk);
					for (nC = 0; nC < AXIS_ALL; nC++)
					{
						if ((StepAmp18 < 4 * thred) || (StateDetectLcVar[2] > PEDOMETER_FREQ))
						{
							StepRecordeState[nC][1] = 0;
							StepRecordTime[nC][1] = 0;
							if (StepRecordTime[nC][3] > StepRecordTime[nB][2])
							{
								StepRecordTime[nC][3] = StepRecordTime[nB][2];
								if (StepRecordeState[nC][3] > StepRecordeState[nB][2])
								{
									StepRecordeState[nC][3] = StepRecordeState[nB][2];
								}
							}
						}
						else if (StepRecordTime[nC][3] >= StepRecordTime[nB][2])
						{
							StepRecordeState[nC][1] = 0;
							StepRecordTime[nC][1] = 0;
							StepRecordTime[nC][3] = StepRecordTime[nB][2];
							if (StepRecordeState[nC][3] > StepRecordeState[nB][2])
							{
								StepRecordeState[nC][3] = StepRecordeState[nB][2];
							}
						}
						else if (StepRecordTime[nC][1] + StepRecordTime[nC][3] >= StepRecordTime[nB][2])
						{
							StepRecordTime[nC][1] = StepRecordTime[nB][2] - StepRecordTime[nC][3];
							if (StepRecordTime[nC][4] > 0)
							{
								StepRecordeState[nC][1] = StepRecordTime[nC][1] * StepRecordTime[nC][4] / StepRecordTime[nC][5];
							}
							else if (StepRecordeState[nC][1] + StepRecordeState[nC][3] >= StepRecordeState[nB][2])
							{
								StepRecordeState[nC][1] = StepRecordeState[nB][2] - StepRecordeState[nC][3];
							}
							else
							{
								StepRecordeState[nC][1] = 0;
							}
						}
					}
				}
				StepRecordeState[FILTER_AXIS_NUM + axis / 4][0] = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][0]   = 0;
				StepRecordeState[FILTER_AXIS_NUM + axis / 4][1] = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][1]   = 0;
				StepRecordeState[FILTER_AXIS_NUM + axis / 4][3] = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][3]   = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][4]   = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][5]   = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][7]   = 0;
				StepRecordTime[FILTER_AXIS_NUM + axis / 4][8]   = 0;
				TimeStampSave[FILTER_AXIS_NUM  + axis / 4][0]   = 0;
			}
		}

		if ((fabs(-FeatureAmp[axis][0] + FeatureAmp[axis][1] - FeatureAmp[axis][2] + FeatureAmp[axis][3]) < FeatureAmp[axis][3] * 0.3l)
			&& (fabs(-FeatureAmp[axis][4] + FeatureAmp[axis][5] - FeatureAmp[axis][6] + FeatureAmp[axis][7]) < FeatureAmp[axis][7] * 0.3l)
			&& (FeatureAmp[axis][3] > thred) && (FeatureAmp[axis][3] > 0.7l * FeatureAmp[axis][2]) && (FeatureAmp[axis][3] > 0.7l * FeatureAmp[axis][1]) && (FeatureAmp[axis][3] > 0.7l * FeatureAmp[axis][0])
			&& (FeatureAmp[axis][7] > thred) && (FeatureAmp[axis][7] > 0.7l * FeatureAmp[axis][6]) && (FeatureAmp[axis][7] > 0.7l * FeatureAmp[axis][5]) && (FeatureAmp[axis][7] > 0.7l * FeatureAmp[axis][4])
			&& (fabs(StepAmp14 - StepAmp58) < 0.15l * StepAmp18)
			&& (ABS_INT(StepTime12 - StepTime56) < 0.15l * StepTime58)
			&& (ABS_INT(StepTime34 - StepTime78) < 0.15l * StepTime58)
			&& (StepTime14 >= StepMinTime)
			&& (StepTime14 < 2 * StepMaxTime)
			&& (StepTime58 > StepMinTime)
			&& (StepTime58 < 2 * StepMaxTime))
		{
			if ((StepTime58 >= StepMaxTime)
				|| ((nD != 0) && (nD < 0.7l * StepTime18 / 4.0l)))
			{
				StepRecordTime[axis][8] = PEDOMETER_FREQ * 8.0 / StepTime18;
			}
			else
			{
				StepRecordTime[axis][8] = PEDOMETER_FREQ * 4.0 / StepTime18;
			}
			TimeStampSave[axis][1] = timeStamp;
			if (StepRecordeState[axis][0] == 0)
			{
				StepRecordeState[axis][0] = StepRecordeState[axis][0] + 4;
				StepRecordTime[axis][0] = StepRecordTime[axis][0] + StepTime18;
				if ((StepTime58 >= StepMaxTime)
					|| ((nD != 0) && (nD < 0.7l * StepTime18 / 4.0l)))
				{
					StepRecordeState[axis][1] = StepRecordeState[axis][1] + 8;
				}
				else
				{
					StepRecordeState[axis][1] = StepRecordeState[axis][1] + 4;
				}
				StepRecordTime[axis][1] = StepRecordTime[axis][1] + StepTime18;
				TimeStampSave[axis][0] = timeStamp - (unsigned long)(StepTime18 * 1000 / PEDOMETER_FREQ);
			}
			else
			{
				// 判断计步器的Amp振幅特征和时间特征是否满足要求
				if (((fabs(FeatureAmp[axis][0] - FeatureAmp[axis][4]) < 0.2l * (FeatureAmp[axis][0] + FeatureAmp[axis][4])) || (fabs(FeatureAmp[axis][0] - FeatureAmp[axis][4]) < 0.075l * StepAmp18))
					&& ((fabs(FeatureAmp[axis][1] - FeatureAmp[axis][5]) < 0.2l * (FeatureAmp[axis][1] + FeatureAmp[axis][5])) || (fabs(FeatureAmp[axis][1] - FeatureAmp[axis][5]) < 0.075l * StepAmp18))
					&& ((fabs(FeatureAmp[axis][2] - FeatureAmp[axis][6]) < 0.2l * (FeatureAmp[axis][2] + FeatureAmp[axis][6])) || (fabs(FeatureAmp[axis][2] - FeatureAmp[axis][6]) < 0.075l * StepAmp18))
					&& ((fabs(FeatureAmp[axis][3] - FeatureAmp[axis][7]) < 0.2l * (FeatureAmp[axis][3] + FeatureAmp[axis][7])) || (fabs(FeatureAmp[axis][3] - FeatureAmp[axis][7]) < 0.075l * StepAmp18))
					&& ((ABS_INT(FeatureTime[axis][0] - FeatureTime[axis][4]) < 0.2l * (FeatureTime[axis][0] + FeatureTime[axis][4])) || (ABS_INT(FeatureTime[axis][0] - FeatureTime[axis][4]) < 5))
					&& ((ABS_INT(FeatureTime[axis][1] - FeatureTime[axis][5]) < 0.2l * (FeatureTime[axis][1] + FeatureTime[axis][5])) || (ABS_INT(FeatureTime[axis][1] - FeatureTime[axis][5]) < 5))
					&& ((ABS_INT(FeatureTime[axis][2] - FeatureTime[axis][6]) < 0.2l * (FeatureTime[axis][2] + FeatureTime[axis][6])) || (ABS_INT(FeatureTime[axis][2] - FeatureTime[axis][6]) < 5))
					&& ((ABS_INT(FeatureTime[axis][3] - FeatureTime[axis][7]) < 0.2l * (FeatureTime[axis][3] + FeatureTime[axis][7])) || (ABS_INT(FeatureTime[axis][3] - FeatureTime[axis][7]) < 5)))
				{
					if ((StepRecordTime[axis][7] > 0) && (fabs(StepTime18 / 4.0 - StepRecordTime[axis][7]) > 0.2 * StepRecordTime[axis][7]))
					{
						StepRecordTime[axis][4] = 0;
						StepRecordTime[axis][5] = 0;
					}
					if (StepRecordTime[axis][6] >= 8)
					{
						StepRecordTime[axis][4] = StepRecordTime[axis][4] + 4;
						StepRecordTime[axis][5] = StepRecordTime[axis][5] + StepTime18;
						StepRecordTime[axis][6] = 0;
					}
					StepRecordTime[axis][7] = StepTime18 / 4.0;
				}
				if (StepRecordeState[axis][3] <= 4)
				{
					StepRecordeState[axis][0] = StepRecordeState[axis][0] + StepRecordeState[axis][3];
					StepRecordTime[axis][0] = StepRecordTime[axis][0] + StepRecordTime[axis][3];
				}
				else
				{
					StepRecordeState[axis][0] = StepRecordeState[axis][0] + 4;
					StepRecordTime[axis][0] = StepRecordTime[axis][0] + StepTime18;
				}
				if ((StepTime58 >= StepMaxTime)
					|| ((nD != 0) && (nD < 0.7l * StepTime18 / 4.0l)))
				{
					if (StepRecordeState[axis][3] <= 4)
					{
						StepRecordeState[axis][1] = StepRecordeState[axis][1] + StepRecordeState[axis][3] * 2;
					}
					else if ((StepRecordTime[axis][3] - StepTime18 > SampleFrequencySix)
						&& (StepRecordeState[axis][3] > 12))
					{
						StepRecordeState[axis][1] = StepRecordeState[axis][1] + 8;//10
					}
					else
					{
						if (StepRecordTime[axis][4] > 0)
						{
							nE = StepRecordTime[axis][3] * StepRecordTime[axis][4] / StepRecordTime[axis][5];
						}
						if ((StepRecordTime[axis][4] > 0) && (nE < StepRecordeState[axis][3]))
						{
							if (nE > 2.5l)
							{
								StepRecordeState[axis][1] = StepRecordeState[axis][1] + nE * 2;
							}
							else
							{
								StepRecordeState[axis][1] = StepRecordeState[axis][1] + 8;//10
							}
						}
						else
						{
							if (StepRecordeState[axis][3] > 2.5l)
							{
								StepRecordeState[axis][1] = StepRecordeState[axis][1] + StepRecordeState[axis][3] * 2;
							}
							else
							{
								StepRecordeState[axis][1] = StepRecordeState[axis][1] + 8;//10
							}
						}
					}

				}
				else
				{
					if (StepRecordeState[axis][3] <= 4)
					{
						StepRecordeState[axis][1] = StepRecordeState[axis][1] + StepRecordeState[axis][3];
					}
					else if ((StepRecordTime[axis][3] - StepTime18 > SampleFrequencySix)
						&& (StepRecordeState[axis][3] > 12))
					{
						StepRecordeState[axis][1] = StepRecordeState[axis][1] + 4;
					}
					else
					{
						if (StepRecordTime[axis][4] > 0)
						{
							nE = StepRecordTime[axis][3] * StepRecordTime[axis][4] / StepRecordTime[axis][5];
						}
						if ((StepRecordTime[axis][4] > 0) && (nE < StepRecordeState[axis][3]))
						{
							if (nE > 5)
							{
								StepRecordeState[axis][1] = StepRecordeState[axis][1] + nE;
							}
							else
							{
								StepRecordeState[axis][1] = StepRecordeState[axis][1] + 4;
							}
						}
						else
						{
							if (StepRecordeState[axis][3] > 5)
							{
								StepRecordeState[axis][1] = StepRecordeState[axis][1] + StepRecordeState[axis][3];
							}
							else
							{
								StepRecordeState[axis][1] = StepRecordeState[axis][1] + 4;
							}
						}
					}

				}
				StepRecordTime[axis][1] = StepRecordTime[axis][1] + StepRecordTime[axis][3];
			}
			StepRecordeState[axis][2] = 0;
			StepRecordTime[axis][2] = 0;
			StepRecordeState[axis][3] = 0;
			StepRecordTime[axis][3] = 0;
		}
		else if ((StepRecordeState[axis][0] > 0) && ((StepRecordTime[axis][2] >= StepMaxTime * 2) || (StepAmp18 < 4 * thred) || (StateDetectLcVar[2] > PEDOMETER_FREQ)))
		{
			nB = AXIS_ALL - 1;
			if (nD > 0)
			{
				for (nC = (AXIS_ALL - 1); nC >= 0; nC--)
				{
					if (StepRecordTime[nB][0] < StepRecordTime[nC][0])
					{
						nB = nC;
					}
				}
			}
			else
			{
				for (nC = (AXIS_ALL - 1); nC >= AXIS_NUM; nC--)
				{
					if ((StepRecordTime[nB][0] < StepRecordTime[nC][0]) && (nC != FILTER_AXIS_NUM))
					{
						nB = nC;
					}
				}
			}
			if ((nB == axis) && (StepRecordeState[nB][0] >= 10))
			{
				if (Steps.walkRealTime > StepRecordeState[nB][1])
				{
					Steps.walk = Steps.walk + Steps.walkRealTime;
				}
				else
				{
					Steps.walk = Steps.walk + StepRecordeState[nB][1];
				}
				Steps.walkRealTime = 0;
				ReSteps.walkSteps = (unsigned long)(Steps.walk);

				for (nC = 0; nC < AXIS_ALL; nC++)
				{
					if ((StepAmp18 < 4 * thred) || (StateDetectLcVar[2] > PEDOMETER_FREQ))
					{
						StepRecordeState[nC][1] = 0;
						StepRecordTime[nC][1] = 0;
						if (StepRecordTime[nC][3] > StepRecordTime[nB][2])
						{
							StepRecordTime[nC][3] = StepRecordTime[nB][2];
							if (StepRecordeState[nC][3] > StepRecordeState[nB][2])
							{
								StepRecordeState[nC][3] = StepRecordeState[nB][2];
							}
						}
					}
					else if (StepRecordTime[nC][3] >= StepRecordTime[nB][2])
					{
						StepRecordeState[nC][1] = 0;
						StepRecordTime[nC][1] = 0;
						StepRecordTime[nC][3] = StepRecordTime[nB][2];
						if (StepRecordeState[nC][3] > StepRecordeState[nB][2])
						{
							StepRecordeState[nC][3] = StepRecordeState[nB][2];
						}
					}
					else if (StepRecordTime[nC][1] + StepRecordTime[nC][3] >= StepRecordTime[nB][2])
					{
						StepRecordTime[nC][1] = StepRecordTime[nB][2] - StepRecordTime[nC][3];
						if (StepRecordTime[nC][4] > 0)
						{
							StepRecordeState[nC][1] = StepRecordTime[nC][1] * StepRecordTime[nC][4] / StepRecordTime[nC][5];
						}
						else if (StepRecordeState[nC][1] + StepRecordeState[nC][3] >= StepRecordeState[nB][2])
						{
							StepRecordeState[nC][1] = StepRecordeState[nB][2] - StepRecordeState[nC][3];
						}
						else
						{
							StepRecordeState[nC][1] = 0;
						}
					}
				}


				StepRecordTime[axis][4] = 0;
				StepRecordTime[axis][5] = 0;


			}
			StepRecordeState[axis][0] = 0;
			StepRecordTime[axis][0] = 0;
			StepRecordeState[axis][1] = 0;
			StepRecordTime[axis][1] = 0;
			StepRecordeState[axis][3] = 0;
			StepRecordTime[axis][3] = 0;
			//	StepRecordeStateTime[nA][4] = 0;
			//	StepRecordeStateTime[nA][5] = 0;
			StepRecordTime[axis][7] = 0;
			StepRecordTime[axis][8] = 0;
			TimeStampSave[axis][0] = 0;
		}

		StepReFeature[axis] = STEP_FALSE;
	}

	nB = AXIS_ALL - 1;
	for (nC = (AXIS_ALL - 1); nC >= 0; nC--){
		if (StepRecordTime[nB][0] < StepRecordTime[nC][0]){
			nB = nC;
		}
	}
	if (StepRecordeState[nB][0] == 0)
	{
		StateLcVar[0] = NonSteps;
		if (Steps.walkRealTime > 0)
		{
			Steps.walk = Steps.walk + Steps.walkRealTime;
			Steps.walkRealTime = 0;
		}
	}

	if (StateLcVar[1] == StateLcVar[0])
	{
		if ((StepRecordTime[nB][1] > StepMinTime) && (StepRecordeState[nB][0] >= 8) && (StepRecordeState[nB][3] == 0))
		{
			if (StateLcVar[1] == walk)
			{
				if (Steps.walkRealTime < StepRecordeState[nB][1])
				{
					Steps.walkRealTime = StepRecordeState[nB][1];

					ReSteps.walkSteps = (unsigned long)(Steps.walk + Steps.walkRealTime);
				}
			}
			else if (Steps.walkRealTime < StepRecordeState[nB][1])
			{
				Steps.walkRealTime = StepRecordeState[nB][1];
				ReSteps.walkSteps = (unsigned long)(Steps.walk + Steps.walkRealTime);
			}
		}
	}
	else
	{
		if ((StepRecordeState[nB][0] >= 8) && (StateLcVar[1] != NonSteps))
		{

			if (Steps.walkRealTime > StepRecordeState[nB][1])
			{
				Steps.walk = Steps.walk + Steps.walkRealTime;
			}
			else
			{
				Steps.walk = Steps.walk + StepRecordeState[nB][1];
			}
			Steps.walkRealTime = 0;
			ReSteps.walkSteps = (unsigned long)(Steps.walk + Steps.walkRealTime);

			for (nC = 0; nC < AXIS_ALL; nC++)
			{
				if (StepRecordTime[nC][3] >= StepRecordTime[nB][2])
				{
					StepRecordeState[nC][1] = 0;
					StepRecordTime[nC][1] = 0;
					StepRecordTime[nC][3] = StepRecordTime[nB][2];
					if (StepRecordeState[nC][3] > StepRecordeState[nB][2])
					{
						StepRecordeState[nC][3] = StepRecordeState[nB][2];
					}
				}
				else if (StepRecordTime[nC][1] + StepRecordTime[nC][3] >= StepRecordTime[nB][2])
				{
					StepRecordTime[nC][1] = StepRecordTime[nB][2] - StepRecordTime[nC][3];
					if (StepRecordTime[nC][4] > 0)
					{
						StepRecordeState[nC][1] = StepRecordTime[nC][1] * StepRecordTime[nC][4] / StepRecordTime[nC][5];
					}
					else if (StepRecordeState[nC][1] + StepRecordeState[nC][3] >= StepRecordeState[nB][2])
					{
						StepRecordeState[nC][1] = StepRecordeState[nB][2] - StepRecordeState[nC][3];
					}
					else
					{
						StepRecordeState[nC][1] = 0;
					}
				}
			}
			StepRecordeState[nB][1] = 0;
			StepRecordTime[nB][1] = 0;
		}
	}

	stateNow.sign = 0;
	if ((StateLcVar[1] != StateLcVar[0]) && ((StateLcVar[0] == NonSteps) || (StepRecordeState[nB][0] >= 8)))
	{
		stateNow.state = StateLcVar[0];
		if (StateLcVar[1] == NonSteps)
		{
			nB = AXIS_ALL - 1;
			for (nC = (AXIS_ALL - 1); nC >= 0; nC--)
			{
				if (((TimeStampSave[nB][0] == 0) || (TimeStampSave[nB][0] > TimeStampSave[nC][0])) && (TimeStampSave[nC][0] > 0))
				{
					nB = nC;
				}
			}
			if (stateNow.stateStartTime < TimeStampSave[nB][0])
			{
				stateNow.stateStartTime = TimeStampSave[nB][0];
			}
			stateNow.stepsStartTime = TimeStampSave[nB][0];
			stateNow.sign = 1;
		}
		else if (StateLcVar[0] == NonSteps)
		{
			nB = AXIS_ALL - 1;
			for (nC = (AXIS_ALL - 1); nC >= 0; nC--)
			{
				if (TimeStampSave[nB][1] < TimeStampSave[nC][1])
				{
					nB = nC;
				}
			}
			if (stateNow.stateStartTime < TimeStampSave[nB][1])
			{
				stateNow.stateStartTime = TimeStampSave[nB][1];
			}
			stateNow.sign = 2;
		}
		else
		{
			stateNow.stateStartTime = timeStamp;
			stateNow.sign = 3;
		}
		StateLcVar[1] = StateLcVar[0];
	}

}

static void StateDetector(unsigned long long timeStamp)
{
	int nA;
	double SumAccX = 0;
	double SumAccYZ = 0;
	double AccX = 0;
	double AccYZ = 0;

	if (StateDetectLcVar[0] < StepMinTime)
	{
		StateDetectLcVar[0] = StateDetectLcVar[0] + 1;
		if (StateDetectLcVar[0] < 0.5l)
		{
			SumAcc = -1;
		}else{
			SumAcc = SumAcc - 1;
		}
	}
	else
	{
		SumAcc = 0;
		for (nA = 0; nA < (ACC_BUFF_NUM - 1); nA++)
		{
			SumAcc += fabs(AccBuffer[3][nA] - AccBuffer[3][nA + 1]);
			SumAccX += fabs(AccBuffer[0][nA] - AccBuffer[0][nA + 1]);
			SumAccYZ += fabs(sqrt(AccBuffer[1][nA] * AccBuffer[1][nA] + AccBuffer[2][nA] * AccBuffer[2][nA])
				- sqrt(AccBuffer[1][nA + 1] * AccBuffer[1][nA + 1] + AccBuffer[2][nA + 1] * AccBuffer[2][nA + 1]));
		}
		SumAcc   /= (ACC_BUFF_NUM - 1);
		SumAccX  /= (ACC_BUFF_NUM - 1);
		SumAccYZ /= (ACC_BUFF_NUM - 1);


		for (nA = 0; nA < ACC_BUFF_NUM; nA++){
			AccX += fabs(AccBuffer[0][nA]);
		}
		AccX = AccX / ACC_BUFF_NUM;

		for (nA = 0; nA < ACC_BUFF_NUM; nA++)
		{
			AccYZ += sqrt(AccBuffer[1][nA] * AccBuffer[1][nA] + AccBuffer[2][nA] * AccBuffer[2][nA]);
		}
		AccYZ /= ACC_BUFF_NUM;

		if (SumAcc > 3 || SumAcc < 0.002l) 
		{
			if (StateDetectLcVar[2] < SampleFrequencyFour)
			{
				StateDetectLcVar[2] = StateDetectLcVar[2] + StepMinTime;
			}
			StateDetectLcVar[3] = 0;
		}
		else
		{
			if (StateDetectLcVar[3] < SampleFrequencyFour)
			{
				StateDetectLcVar[3] = StateDetectLcVar[3] + StepMinTime;
			}
			StateDetectLcVar[2] = 0;
		}

		if (StateDetectLcVar[2] > SampleFrequencydouble)
		{

			StateDetectLcVar[4] = 1;
		}
		else if (StateDetectLcVar[2] > SampleFrequencyOPF)
		{
			STEPLIB_MEMSET(StepReFeature, 1, sizeof(StepReFeature));
		}

		if (StateDetectLcVar[3] > StepMinTime)
		{
			StateDetectLcVar[4] = 0;
		}

		if (((AccX * 1.3l > AccYZ) || (SumAccX * 1.3l > SumAccYZ)) && (SumAcc < 0.8l))
		{
			if (StateDetectLcVar[1] > 0)
			{
				StateDetectLcVar[1] = StateDetectLcVar[1] - 1;
			}else{
				StateDetectLcVar[1] = 0;
			}
		}
		else
		{
			if ((StateDetectLcVar[1] > 4) || (SumAcc > 0.6l))
			{
				StateDetectLcVar[1] = 8;//10
			}
			else
			{
				if ((AccX < AccYZ) && (SumAcc > 0.25l))
				{
					if (StateDetectLcVar[1] < 8)//10
					{
						StateDetectLcVar[1] = StateDetectLcVar[1] + 1;
					}
				}
				else
				{
					StateDetectLcVar[1] = 0;
				}
			}
		}

		if (StateDetectLcVar[1] > 4){
			StateLcVar[0] = 2;
		}else{
			StateLcVar[0] = walk;
		}
		StateDetectLcVar[0] = 0;
	}
}
void Pedometer_GetState(StepState_t* state){
	*state = StateLcVar[0];
}