/*********************************************************************************/
/* FILE NAME  : scene_recognition.h                                                    */
/* AUTHOR     : Zhang Jingrui, ID: 11082826, Version: V0.1_Beta, Data:2019-03-28       */
/* DESCRIPTION: This header file conterns the prototype description of the external    */
/*              interface funtions of the scene recognition module and the definit-    */
/*              ion of related resources.                                              */
/* HISTORY    : (NONE)                                                                 */
/********************************************************************************/

#ifndef _PDR_SCENE_RECOGNITION_H_
#define _PDR_SCENE_RECOGNITION_H_

/* Header Files Including -----------------------------------------*/
/*=================================================================*/
#include "pdr_sensor.h"

/* Macro Declaration ----------------------------------------------*/
/*=================================================================*/

#define _TIME_BUFFER_SIZE               10      /* Buffer size for the time calculation.                   */
#define _ACCURACY_BUFFER_SIZE           9       /* Accuracy time-based buffer size.                        */
#define _GNSS_INDEX_LEN                 103     /* Structural array size in _GnssInfo.                     */

/* Extern Variable Declaration ------------------------------------*/
/*=================================================================*/
//extern int scene_recognition_debug_flag ;
//extern int scene_recognition_log_debug_flag ;

/* Status Declaration ---------------------------------------------*/
/*=================================================================*/
/* Usage: ʶģʼò״̬־λ                   */
typedef enum SCENE_INIT_STATE
{
    // ʶģʼ״̬
            INIT_SUCCESS = 0,
    INIT_NOT_PERFORMED,

    INIT_ERROR_MEMORY_OCCUPIED,     // ģԴδǰͷţͷԴ
    INIT_ERROR_MEMORY_INIT_FAILED,  // ʼڴʧ
    INIT_ERROR_REDUNDANCY,          // ģظʼʱʼЧҪģ飬ԣ
    // 1ȹرģ飬رճɹ³ʼģ顣
    // 2ģ飬Чͬ1
    // ʶģ״̬(for debug)
            INIT_RESET_SUCCESS = 0,
    INIT_RESET_FAILED
} SCENE_INIT_STATE;

/* Usage:                                             */
typedef enum SCENE_RECOGNITION_RESULT
{
    RECOG_UNKNOWN,                  // δȷ״̬(Ĭ״̬)
    RECOG_OPEN_AREA,                // open-area
    RECOG_OTHERS,                   // ȷ
    RECOG_ERROR_PARAM_INCRECT,      // 
    RECOG_ERROR_NMEA_INFO_MISS,     // ȷ
    /* Ԥм(ȶ) */
            RECOG_MULTIPATH             // ྶ
} SCENE_RECOGNITION_RESULT;

/* Usage: ʶģԴͷŲ                             */
typedef enum SCENE_DESTROY_STATE
{
    DESTROY_SUCCESS = 0,
    DESTROY_FIALED,
    DESTROY_ERROR,
    DESTROY_INVALID     // ЧͷŲʾûжӦĳʼò
} SCENE_DESTROY_STATE;


/* Status Declaration--------------------------------------------*/
/*===============================================================*/

// ǰʶģĹ״̬
typedef enum _SCENE_MODEL_WORK_STATE
{
    MODEL_OFF,
    MODEL_ON
} _SCENE_MODEL_WORK_STATE;

/* Usage: Identifier of the quality of the satellites' signal source. */
typedef enum _SIGNAL_QUALITY
{
    SIG_UNKNOWN,
    GOOD,
    BAD
} SIGNAL_QUALITY;

/* Usage: extract and save information from lct_nmea for the module. */
/*        This is the interface for the lct_nmea.                    */
typedef struct _GnssInfo
{
    int    update;                                         // GNSS±־λ 1 0
    double local_timestamp;                                // ʱ
    float  accuracy;                                       // GPS AccuracyAccuracy > 0 ΪʾЧ
    int    sat_visible_number;                             // ǰʱɼ
    float  snr_list[_GNSS_INDEX_LEN];               // ǰʱɼǵSNRб
    int    sat_used_list[_GNSS_INDEX_LEN];               // ǰʹб
} GnssInfo;

/* Usage: GNSSеĸϵͳ                                 */

/* Struct Declaration -------------------------------------------*/
/*===============================================================*/


/* Function Declaration -------------------------------------------*/
/*=================================================================*/
#ifdef __cplusplus
extern "C" {
#endif

/* Function Name: sceneRecognitionInit()                   */
/* Usage: Initialization module for configuring related resources  */
/*        and must be executed once first beforing using the scene */
/*        recognition module funcition.                            */
/* Param  @ (NONE)                                                 */
/* Return @ init_res: Result state of the initialization process.  */
SCENE_INIT_STATE initSceneRecognition(void);

/* Function Name: scene_recognition_reset()                      */
/* Usage: ʼģ飬ԴʹģǰȱִС      */
/* Param  @ (NONE)                                               */
/* Return @ init_res: ִн                              */
SCENE_INIT_STATE scene_recognition_reset(void);

/**----------------------------------------------------------------------
* Function    : sceneRecognitionProc
* Description : GNSSʶ̣
*               1nmeaתΪgnss
*               2) ͨaccuracysnrGNSSź
*               3) ؽ
* Input       : PDRnmeaṹ
* Return      : ʶ
* Date        : 2020/02/18  yuanlin_rjyb@vivo.com & zhanli@vivo.com
*---------------------------------------------------------------------**/
SCENE_RECOGNITION_RESULT sceneRecognitionProc(const lct_nmea* nmea);

/* Function Name: scene_recognition_destroy()                    */
/* Usage: ʶԴͷŲԴͷź޷г  */
/*        ʶ̡                                            */
/* Param  @ (NONE)                                               */
/* Return @ destroy_state: ԴͷŲ                      */
SCENE_DESTROY_STATE scene_recognition_destroy(void);

/**----------------------------------------------------------------------
* Function    : isOpenArea
* Description : GNSSؼ⣬ͨGNSSźţǰźӶȷ
*               ĿǷڿء
* Return      : 0 : ǿ  1(źϺ)
* Author      : Zhang Jingrui, VIVO ID: 11082826
* Date        : 2020/02/18  yuanlin_rjyb@vivo.com & zhanli@vivo.com
*---------------------------------------------------------------------**/
int isOpenArea(const lct_nmea* nmea);

#ifdef __cplusplus
}
#endif
#endif  //__SCENE_RECOGNITION_H__